#!/usr/bin/env php -q
<?php

	// Script para sincronizar arquivos na pasta "orders" do projeto AnaBlock
	// com o banco de dados

	// Funcoes rapidas
	require('/anablock/lib/phpb.php');
	require('/anablock/lib/usql.php');
	require('/anablock/lib/libconf.php');
	require('/anablock/lib/vlib.php');
	require('/anablock/lib/ipv4.php');
	require('/anablock/lib/ipv6.php');

	function date_read_to_unix_timestamp($usdate=''){
		// Quebrar yyyy-mm-dd-hh:mm:ss pelo hifem
		$a = explode('-', trim($usdate));
		if(!isset($a[0])) $a[0] = 0; // ano
		if(!isset($a[1])) $a[1] = 0; // mes
		if(!isset($a[2])) $a[2] = 0; // dia
		if(!isset($a[3])) $a[3] = '00:00:00'; // h:m:s
		// A quarta parte e' a hora/min/seg
		$h = explode(':', $a[3]);
		if(!isset($h[0])) $h[0] = 0; // hora
		if(!isset($h[1])) $h[1] = 0; // minuto
		if(!isset($h[2])) $h[2] = 0; // segundo
		// Construir timestamp
		$uts = mktime(
			$h[0], // hora
			$h[1], // minuto
			$h[2], // segundo
			$a[1], // mes
			$a[2], // dia
			$a[0]  // ano
		);
		return $uts;
	}

	// listas gerais
	$alldomains = array();

	echo "# Iniciando sincronismo de dados com o banco de dados\n";
	
	// Variaveis para conexao com MariaDB
	usql_connect();

	// Arquivos esperados
	$ofile_names = array('domains.txt', 'urls.txt', 'ipv4.txt', 'ipv6.txt', 'asn.txt');

	// Log binario dos dados manipulados
	$logbin = '';

	// Diretorios dos arquivos das ordens
	$dlist =& _dir_folders('/anablock/orders');
	sort($dlist);
	foreach($dlist as $k=>$order_dir){
		$oid = (int)basename($order_dir);
		echo "# - [$oid] Analisando diretorio: ",$order_dir," ";


		// Manifesto da ordem
		$ocfg = $order_dir.'/order.conf';

		// Manifesto nao existe
		if(!is_file($ocfg)){
			echo "[ignorando, sem manifesto order.conf]\n";
			continue;
		}

		// Carregar manifesto
		$order_config = libconf_load($ocfg);
		// - Critica basica
		if($order_config['name']==''){
			echo "[ignorando, ordem sem nome]\n";
			continue;
		}

		// Converter tipos
		$order_config['otype']        = (int)$order_config['otype'];
		$order_config['confidential'] = (int)$order_config['confidential'];

		// Data para timestamp
		// $order_config['ts_created'] = strtotime($order_config['created']);
		// $order_config['ts_start']   = strtotime($order_config['start']);
		// $order_config['ts_expire']  = strtotime($order_config['expire']);
		$order_config['ts_created'] = date_read_to_unix_timestamp($order_config['created']);
		$order_config['ts_start']   = date_read_to_unix_timestamp($order_config['start']);
		$order_config['ts_expire']  = date_read_to_unix_timestamp($order_config['expire']);

		if($oid == 20230728001){
			print_r($order_config);
			//exit();
		}

		// Colunas do registro na tabela
		$_reg = array(
			'id' => $oid,
			'name' => $order_config['name'],
			'comment' => $order_config['comment'],
			'otype' => $order_config['otype'],
			'confidential' => $order_config['confidential'],
			'ts_created' => $order_config['ts_created'],
			'ts_start' => $order_config['ts_start'],
			'ts_expire' => $order_config['ts_expire']
		);

		// Atualizar ou inserir? Pesquisar primeiro
		$sql = "SELECT * FROM anablock_order WHERE id = ".$oid;
		$lst = usql_select($sql, 'id');
		if(count($lst)){
			// Ja existe, sincronizar colunas
			// - nao pode fazer update do id
			unset($_reg['id']);
			$sqlu = usql_reg2u($_reg, 'anablock_order') . ' WHERE id = '.$oid.' LIMIT 1;';
			$ru = usql_query($sqlu);
			if(!$ru){
				echo "# Fatal, erro ",$USQL_ERRNO,"=",$USQL_ERROR," ao executar:\n\t",$sqlu,"\n";
				exit();
			}
			echo "(atualizado $oid)\n";
			$logbin .= serialize($_reg);

		}else{

			// Nao existe, inserir
			echo "(inserir $oid)";
			// - Gerar SQL de INSERT usando o array
			$_reg['id'] = $oid;
			$sqli = usql_a2i($_reg, 'anablock_order');
			$ri = usql_query($sqli);
			if(!$ri){
				echo "# Fatal, erro ",$USQL_ERRNO,"=",$USQL_ERROR," ao executar:\n\t",$sqli,"\n";
				exit();
			}
			$logbin .= serialize($_reg);
			echo "[inserido]\n";

		}

		// Analisar listas TXT para sincronizar objetos manifestos


		// domains.txt
		echo "# - [$oid] - Sincronizando domains.txt ";
		$_txt = $order_dir.'/domains.txt';
		$_list = array();
		if(is_file($_txt)){
			// Limpar lista
			$_lines = file($_txt);
			foreach($_lines as $k=>$line){
				$line = trim($line);
				$line = strtolower($line);

				// Ignorar linha vazia
				if($line==''){ unset($_lines[$k]); continue; }

				// Ignorar comentarios
				if(substr($line, 0, 1)=='#'){ unset($_lines[$k]); continue; }

				// Dominios sintaticamente incorretos devem abortar a compilacao
				if(!vlib_fqdn($line)){
					echo "# ERRO, nome de dominio sintaticamente incorreto: [$line]\n";
					exit();
				}
				$_list[$k] = $line;

				// lista unificada
				$alldomains[$line] = $k;
			}
			sort($_list);
		}
		// Limpar
		usql_query("DELETE FROM anablock_objects_domain WHERE order_id = ".$oid);
		// Registrar lista
		$ct_domains = 0;
		foreach($_list as $k=>$_domain){
			$_newreg = array(
				'order_id' => $oid,
				'fqdn' => $_domain,
				'ts_created' => $order_config['ts_created'],
				'ts_start' => $order_config['ts_start'],
				'ts_expire' => $order_config['ts_expire']
			);
			$sqli = usql_a2i($_newreg, 'anablock_objects_domain');
			$ri = usql_query($sqli);
			if(!$ri){
				echo "# Fatal, erro ",$USQL_ERRNO,"=",$USQL_ERROR," ao executar ",$sqli,"\n";
				exit();
			}
			$logbin .= serialize($_newreg);
			$ct_domains++;
			echo ".";
		}
		echo "\n";



		// urls.txt
		echo "# - [$oid] - Sincronizando urls.txt ";
		$_txt = $order_dir.'/urls.txt';
		$_list = array();
		if(is_file($_txt)){
			// Limpar lista
			$_lines = file($_txt);
			foreach($_lines as $k=>$line){
				$line = trim($line);
				$line = strtolower($line);
				if($line==''){ unset($_lines[$k]); continue; }
				$_list[$k] = $line;
			}
			sort($_list);
		}
		// Limpar
		usql_query("DELETE FROM anablock_objects_url WHERE order_id = ".$oid);
		// Registrar lista
		$ct_urls = 0;
		foreach($_list as $k=>$_url){
			// extrair o dominio da URL
			$_doamin = $_url;
			$_domain = str_replace('https://', '', $_doamin);
			$_domain = str_replace('http://', '', $_doamin);
			$a = explode('/', $_doamin);
			$_domain = $a[0];
			$_newreg = array(
				'order_id' => $oid,
				'url' => $_url,
				'fqdn' => $_domain,
				'ts_created' => $order_config['ts_created'],
				'ts_start' => $order_config['ts_start'],
				'ts_expire' => $order_config['ts_expire']
			);
			$sqli = usql_a2i($_newreg, 'anablock_objects_url');
			$ri = usql_query($sqli);
			if(!$ri){
				echo "# Fatal, erro ",$USQL_ERRNO,"=",$USQL_ERROR," ao executar ",$sqli,"\n";
				exit();
			}
			$logbin .= serialize($_newreg);
			$ct_urls++;
			echo ".";
		}
		echo "\n";


		// IPv4
		echo "# - [$oid] - Sincronizando ipv4.txt ";
		$_txt = $order_dir.'/ipv4.txt';
		$_list = array();

		if(is_file($_txt)){
			// Limpar lista
			$_lines = file($_txt);
			foreach($_lines as $k=>$line){
				$line = trim($line);
				$line = strtolower($line);
				if($line=='' || substr($line, 0, 1)=='#'){ unset($_lines[$k]); continue; }

				// Converter em prefixo
				$p = strpos($line, '/');
				$bits = 32;
				$addr = $line;
				if($p===false){
					// sem mascara
					$bits = 32;
				}else{
					// com mascara
					$addr = substr($line, 0, $p);
					$bits = (int)substr($line, $p+1);
				}
				if($bits < 0 || $bits > 32){
					// Como isso foi possivel?
					echo "# ERRO (1), endereco ipv4 ou prefixo sintaticamente incorreto: [$line] [$addr] [$bits]\n";
					exit();
				}

				// Teste sintatico
				if(!vlib_ipv4($addr)){
					echo "# ERRO (2), endereco ipv4 ou prefixo sintaticamente incorreto: [$line] [$addr] [$bits]\n";
					exit();
				}

				// Carregar endereco para calculo de subrede
				$ipinfo =& ipv4_load($addr, $bits);
				$ipcount = $ipinfo['bcount'];
				$prefix  = $ipinfo['prefix'];
				$prefix_start = ip2long($ipinfo['network']);
				$prefix_end   = ip2long($ipinfo['broadcast']);

				echo "# Info: line[$line] prefix[$prefix] addr[$addr] bits[$bits] ipcount[$ipcount] prefix_start=[$prefix_start] prefix_end=[$prefix_end]\n";

				$_newreg = array(
					'order_id' => $oid,
					'asn' => 0,
					'prefix' => $prefix,
					'plen' => $bits,
					'pcount' => $ipcount,
					'prefix_start' => $prefix_start,
					'prefix_end' => $prefix_end,
					'ts_created' => $order_config['ts_created'],
					'ts_start' => $order_config['ts_start'],
					'ts_expire' => $order_config['ts_expire']
				);
				$_list[$prefix_start] = $_newreg;

			}
			ksort($_list);
		}
		// Limpar
		usql_query("DELETE FROM anablock_objects_prefix4 WHERE order_id = ".$oid);
		// Registrar lista
		$ct_ipv4 = 0;
		foreach($_list as $idx=>$addr_reg){
			$sqli = usql_a2i($addr_reg, 'anablock_objects_prefix4');
			$ri = usql_query($sqli);
			if(!$ri){
				echo "# Fatal, erro ",$USQL_ERRNO,"=",$USQL_ERROR," ao executar ",$sqli,"\n";
				exit();
			}
			$logbin .= serialize($addr_reg);
			$ct_ipv4++;
			echo ".";
		}
		echo "\n";


		// IPv6 - Falta
		$ct_ipv6 = 0;

		// ASN  - Falta
		$ct_asn = 0;



		// Atualizar contadores
		$_counters = array(
			'ct_domains' => $ct_domains,
			'ct_urls' => $ct_urls,
			'ct_ipv4' => $ct_ipv4,
			'ct_ipv6' => $ct_ipv6,
			'ct_asn' => $ct_asn
		);
		$sqlu = usql_reg2u($_counters, 'anablock_order') . ' WHERE id = '.$oid.' LIMIT 1;';
		$ru = usql_query($sqlu);
		$logbin .= serialize($_counters);

		echo "# - [$oid] - Concluido, contadores: domains=$ct_domains, urls=$ct_urls, ipv4=$ct_ipv4, ipv6=$ct_ipv6, asn=$ct_asn\n";

		echo "\n\n";

	}

	// ARquivo com timestamp da versao
	$vts_file = '/anablock/htdocs/version.ts';

	// MD5 dos dados processados atualmente
	$act_md5sum = md5($logbin);

	// Obter MD5 anterior
	$md5sigfile = '/anablock/htdocs/logbin.md5';
	$old_md5sum = trim(@file_get_contents($md5sigfile));


	echo "# Assinatura atual....: $act_md5sum\n";
	echo "# Assinatura anterior.: $old_md5sum\n";


	// Se mudou, incrementar o timestamp
	if($act_md5sum != $old_md5sum){
		echo "# Assinatura modificado, atualizar serial version\n";
		$version_ts = time();
		file_put_contents($vts_file, $version_ts);
		file_put_contents($md5sigfile, $act_md5sum);
	}else{
		$version_ts = (int)@file_get_contents($vts_file);
		echo "# Assinatura persistente\n";
	}
	echo "# Versao da base de dados: $version_ts\n";

	// Gravar listas unificadas
	$_alldomains_list = '/anablock/htdocs/alldomains.txt';
	$_alldomains_count = '/anablock/htdocs/alldomains.count';
	ksort($alldomains);
	$c = count($alldomains);
	echo "# Gravando lista de dominios unificados: ",$c," dominios [",$_alldomains_list,"] [",$_alldomains_count,"]\n";
	file_put_contents($_alldomains_list, implode("\n", array_keys($alldomains))."\n" );
	file_put_contents($_alldomains_count, $c."\n" );

	echo "# Finalizado\n";





?>