#!/bin/sh


# Preparar FRR em Debian (11 ou 12)


# FRR - Setup
#==================================================================================================





# Instalar FRR para RouterServer / RouterReflector na nuvem
#--------------------------------------------------------------------------------------------------

    apt-get -y install frr frr-rpki-rtrlib frr-snmp

    # Ativar servico BGPd
    sed  -i  '/bgpd/s/=no/=yes/'    /etc/frr/daemons

    # sed  -i  '/ospfd/s/=no/=yes/'    /etc/frr/daemons
    # sed  -i  '/ospf6d/s/=no/=yes/'   /etc/frr/daemons
    # sed  -i  '/ripd/s/=no/=yes/'     /etc/frr/daemons
    # sed  -i  '/ripngd/s/=no/=yes/'   /etc/frr/daemons
    # sed  -i  '/isisd/s/=no/=yes/'    /etc/frr/daemons
    # sed  -i  '/pimd/s/=no/=yes/'     /etc/frr/daemons
    # sed  -i  '/pim6d/s/=no/=yes/'    /etc/frr/daemons
    # sed  -i  '/ldpd/s/=no/=yes/'     /etc/frr/daemons
    # sed  -i  '/nhrpd/s/=no/=yes/'    /etc/frr/daemons
    # sed  -i  '/eigrpd/s/=no/=yes/'   /etc/frr/daemons
    # sed  -i  '/babeld/s/=no/=yes/'   /etc/frr/daemons
    # sed  -i  '/sharpd/s/=no/=yes/'   /etc/frr/daemons
    # sed  -i  '/pbrd/s/=no/=yes/'     /etc/frr/daemons
    # sed  -i  '/bfdd/s/=no/=yes/'     /etc/frr/daemons
    # sed  -i  '/fabricd/s/=no/=yes/'  /etc/frr/daemons
    # sed  -i  '/vrrpd/s/=no/=yes/'    /etc/frr/daemons
    # sed  -i  '/pathd/s/=no/=yes/'    /etc/frr/daemons


    # Ativar modo memory-only (nao instala as rotas na FIB do kernel)
    egrep -q 'bgpd_options.*kernel' /etc/frr/daemons || {
        sed -i 's#bgpd_options="#bgpd_options="--no_kernel #' /etc/frr/daemons
    }

    # Ativar frr durante o boot
    systemctl enable frr.service


    # Deixar FRR pronto e rodando:
    service frr restart




# Configurar FRR
#--------------------------------------------------------------------------------------------------


    # Ativar modo de config unificada:
    (
        echo 'config t'
        echo 'frr defaults traditional'
        echo 'log syslog informational'
        echo 'service integrated-vtysh-config'
        echo 'end'
        echo 'write'
    ) | vtysh


    # BOOT do BGP - AS privado: 65534
    #echo '    bgp graceful-restart notification'
    #echo '    bgp allow-martian-nexthop'
    (
        echo 'config t'
        echo 'router bgp 65534'
        echo '    bgp router-id 198.18.255.254'
        echo '    no bgp default ipv4-unicast'
        echo '    no bgp ebgp-requires-policy'
        echo '    no bgp network import-check'
        echo '    bgp bestpath compare-routerid'
        echo '    bgp log-neighbor-changes'
        echo '    bgp bestpath as-path multipath-relax'
        echo '    bgp graceful-shutdown'
        echo '    bgp route-reflector allow-outbound-policy'
        echo 'end'
        echo 'write'
    ) | vtysh

 
    # Peering aberto para todo o mundo!
    # - policy
    (
        echo 'config t'
        echo
        # exportar base de rotas bgp
        echo 'route-map V4FRIENDS-EXPORT-IPV4 permit 65535'
        echo '!'
        echo 'route-map V4FRIENDS-EXPORT-IPV6 permit 65535'
        echo '!'
        echo
        # nao importar nada
        echo 'route-map V4FRIENDS-IMPORT-IPV4 deny 65535'
        echo '!'
        echo 'route-map V4FRIENDS-IMPORT-IPV6 deny 65535'
        echo '!'
        echo
        echo 'end'
        echo 'write'
    ) | vtysh

    # - peerings
    (
        echo 'config t'
        echo
        echo 'router bgp 65534'
        echo '    neighbor V4FRIENDS peer-group'
        echo '    neighbor V4FRIENDS remote-as external'
        echo '    neighbor V4FRIENDS description Export-BH-ROUTES-IPv4'
        echo '    neighbor V4FRIENDS passive'
        echo '    neighbor V4FRIENDS ebgp-multihop 255'
        echo '    neighbor V4FRIENDS enforce-first-as'
        echo '    neighbor V4FRIENDS timers 10 240'
        echo '    neighbor V4FRIENDS graceful-restart'
        echo '    bgp listen range 0.0.0.0/0 peer-group V4FRIENDS'
        echo
        echo '    address-family ipv4 unicast'
        echo '        neighbor V4FRIENDS activate'
        echo '        neighbor V4FRIENDS default-originate'
        echo '        neighbor V4FRIENDS route-map V4FRIENDS-EXPORT-IPV4 out'
        echo '        neighbor V4FRIENDS route-map V4FRIENDS-IMPORT-IPV4 in'
        echo '    exit-address-family'
        echo
        echo 'end'
        echo 'write'
    ) | vtysh





# FRR - Visualizacao
#==================================================================================================


# Entre no shell do frr:
    vtysh



# Comandos:
     show   bgp  ipv4                                   # Listar todo banco de dados IPv4
     show   bgp  ipv6                                   # Listar todo banco de dados IPv6
     show   bgp  summary                                # Listar peerings - todos
     show   bgp  ipv4 summary                           # Listar peerings feitos com TCP/IPv4
     show   bgp  ipv6 summary                           # Listar peerings feitos com TCP/IPv6
     show bgp ipv4 neighbors 10.7.7.1                   # RelatÃ³rio do peering
     show bgp ipv4 neighbors 10.7.7.1 flap-statistics   # RelatÃ³rio de flaps no peering
     show bgp ipv4 neighbors 10.7.7.1 prefix-counts     # Contagem de prefixos no peering
     show bgp ipv4 neighbors 10.7.7.1 advertised-routes # Prefixos IPv4 enviados
     show bgp ipv4 neighbors 10.7.7.1 received-routes   # Prefixos IPv4 recebidos
     show bgp ipv4 neighbors 10.7.7.1 routes            # Rotas IPv4 operacionais
     clear bgp *                                        # Reinicia e zera todos os peerings
     clear bgp 10.7.7.1                                 # Reinicia e zera o peering
     clear bgp 10.7.7.1 in                              # Reinicia rotas recebidas
     clear bgp 10.7.7.1 out                             # Reinicia rotas enviadas
     clear bgp 10.7.7.1 soft                            # Reinicia o peering sem quebrar adjacÃªncia
     clear bgp 10.7.7.1 soft in                         # Reinicia rotas recebidas sem zerar atuais
     clear bgp 10.7.7.1 soft out                        # Reinicia rotas enviadas sem zerar atuais









# Lixao:
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================
#==================================================================================================



# Cliente de teste
#==================================================================================================




    # Ativar modo de config unificada:
    ( echo 'config t'; echo 'service integrated-vtysh-config'; echo 'end'; ) | vtysh
    ( echo 'write'; ) | vtysh

    # AS publico cliente: 61613
    (
        echo 'config t'
        echo 'router bgp 61613'
        echo '    bgp router-id 45.171.60.0'
        echo '    no bgp default ipv4-unicast'
        echo '    no bgp ebgp-requires-policy'
        echo '    no bgp network import-check'
        echo '    bgp log-neighbor-changes'
        echo '    bgp bestpath as-path multipath-relax'
        echo '    bgp graceful-restart notification'
        echo '    bgp graceful-shutdown'
        echo '    bgp allow-martian-nexthop'
        echo '    bgp route-reflector allow-outbound-policy'
        echo 'end'
        echo 'write'
    ) | vtysh

    # Politica de recebimento - aceitar e bloquear
    (
        echo 'config t'
        echo 'route-map BLACKHOLE-IMPORT-IPV4 permit 65535'
        echo 'end'
        echo 'write'
    ) | vtysh

    # Politica de envio - nao enviar nada
    (
        echo 'config t'
        echo 'route-map BLACKHOLE-EXPORT-IPV4 deny 65535'
        echo 'end'
        echo 'write'
    ) | vtysh

    # Rota estatica para eBGP multihop
    (
        echo 'config t'
        echo '    ip route 172.233.24.85/32 170.254.137.65 1'
        echo 'end'
        echo 'write'
    ) | vtysh


    # Peering aberto para todo o mundo!
    (
        echo 'config t'
        echo 'router bgp 61613'
        echo '    neighbor 172.233.24.85 remote-as external'
        echo '    neighbor 172.233.24.85 advertisement-interval 0'
        echo '    neighbor 172.233.24.85 timers connect 1'
        echo '    neighbor 172.233.24.85 ebgp-multihop 250'
        echo '    neighbor 172.233.24.85 solo'
        echo '    neighbor 172.233.24.85 update-source 170.254.137.69'
        echo '    neighbor 172.233.24.85 tcp-mss 1200'
        echo '    no neighbor 172.233.24.85 shutdown'
        echo '    address-family ipv4 unicast'
        echo '        neighbor 172.233.24.85 activate'
        echo '        neighbor 172.233.24.85 soft-reconfiguration inbound'
        echo '        neighbor 172.233.24.85 send-community all'
        echo '        neighbor 172.233.24.85 route-map BLACKHOLE-IMPORT-IPV4 in'
        echo '        neighbor 172.233.24.85 route-map BLACKHOLE-EXPORT-IPV4 out'
        echo 'end'
        echo 'write'
    ) | vtysh

























    # 0-600
    # bgp route-map delay-timer 10
    # 172.233.24.85
    # 2600:3c0d::f03c:93ff:fe26:8fd9

        neighbor 172.233.24.85 remote-as external
        neighbor 172.233.24.85 advertisement-interval 0
        neighbor 172.233.24.85 timers connect 1
        neighbor 172.233.24.85 ttl-security hops 250
        neighbor 172.233.24.85 solo
        neighbor 172.233.24.85 update-source 191.37.79.225
        neighbor 172.233.24.85 tcp-mss 1200
        neighbor 10.7.7.1 password TULIPA2021
        neighbor 10.7.7.1 shutdown
        neighbor 172.16.1.1 ebgp-multihop 250

        address-family ipv4 unicast
            neighbor 172.233.24.85 activate
            neighbor 172.233.24.85 allowas-in 1
            neighbor 172.233.24.85 route-reflector-client
            neighbor 172.233.24.85 soft-reconfiguration inbound
            neighbor 172.233.24.85 send-community all
            neighbor 172.233.24.85 route-map ALLOW-ALL in
            neighbor 172.233.24.85 route-map ALLOW-ALL out
            neighbor 10.7.7.2 remove-private-AS all

        address-family ipv6 unicast


    neighbor peer-group fabric
    neighbor fabric remote-as external
    neighbor swp1 interface peer-group fabric
    neighbor swp2 interface peer-group fabric
    neighbor swp3 interface peer-group fabric
    neighbor swp4 interface peer-group fabric
    neighbor swp5 interface peer-group fabric
    neighbor swp6 interface peer-group fabric

    address-family ipv4 unicast
        neighbor fabric activate
        redistribute connected route-map LOOPBACKS
    address-family l2vpn evpn
        neighbor fabric activate
!
route-map LOOPBACKS permit 10
    match interface lo
!


#------------------------------------------------------------------


    router bgp 65053
        bgp router-id 10.0.0.53
        bgp bestpath as-path multipath-relax
        neighbor fabric peer-group
        neighbor fabric remote-as external
        neighbor eth1.10 interface peer-group fabric
        neighbor eth1.11 interface peer-group fabric
        address-family ipv4 unicast
            neighbor fabric activate
            redistribute connected route-map LOOPBACKS
    !
    route-map LOOPBACKS permit 10
        match interface lo
    !


# The FRR configuration for leaf01 is as follows:

    router bgp 65011
        bgp router-id 10.0.0.11
        bgp bestpath as-path multipath-relax
        neighbor fabric peer-group
        neighbor fabric remote-as external
        neighbor swp51 interface peer-group fabric
        neighbor swp52 interface peer-group fabric
        address-family ipv4 unicast
            neighbor fabric activate
            redistribute connected route-map LOOPBACKS
        !
        address-family l2vpn evpn
            neighbor fabric activate
            advertise-all-vni

!
    route-map LOOPBACKS permit 10
        match interface lo
!



# exit01âs FRR configuration looks as follows:

    router bgp 65041
        bgp router-id 10.0.0.41
        bgp bestpath as-path multipath-relax
        neighbor fabric peer-group
        neighbor fabric remote-as external
        neighbor swp51 interface peer-group fabric
        neighbor swp52 interface peer-group fabric
        neighbor 10.253.0.1 interface remote-as external
        address-family ipv4 unicast
            neighbor fabric activate
            neighbor swp1.3 allowas-in 1
            redistribute connected route-map LOOPBACKS
        !
        address-family l2vpn evpn
            neighbor fabric activate
            advertise-all-vni
            advertise-default-gw
    !
    route-map LOOPBACKS permit 10
        match interface lo
    !
    router bgp 65041 vrf internet-vrf
        bgp router-id 10.0.0.41
        bgp bestpath as-path multipath-relax
        neighbor internet peer-group
        neighbor internet remote-as external
        neighbor 10.253.1.1 interface peer-group internet
        neighbor swp44 interface peer-group internet
        address-family ipv4 unicast
            neighbor internet activate
            neighbor 10.253.1.1 allowas-in 1
            redistribute connected route-map INTERNET
    !
    route-map INTERNET permit 10
        match interface internet-vrf
    !





# exit01âs configuration looks as follows:

    vrf evpn-vrf
       vni 104001
    !
    ! default VRF peering with the underlay and the firewall
!
router bgp 65041
    bgp router-id 10.0.0.41
    bgp bestpath as-path multipath-relax
    neighbor fabric peer-group
    neighbor fabric remote-as external
    neighbor swp51 interface peer-group fabric
    neighbor swp52 interface peer-group fabric
    neighbor swp1.2 interface remote-as external
    address-family ipv4 unicast
        neighbor fabric activate
        neighbor swp1.2 allowas-in 1
        redistribute connected route-map LOOPBACKS
    !
    address-family l2vpn evpn
        neighbor fabric activate
        advertise-all-vni
    !
!
route-map LOOPBACKS permit 10
    match interface lo
!
! evpn vrf peering to announce default route to internal net
!
router bgp 65041 vrf evpn-vrf
    bgp router-id 10.0.0.41
    neighbor swp1.3 interface remote-as external
    address-family ipv4 unicast
       network 10.1.3.0/24
       network 10.2.4.0/24
       neighbor swp1.3 allowas-in 1
    exit-address-family
    !
    ! This config ensures we advertise the default route
    ! as a type 5 route in EVPN in the main BGP instance.
    ! The firewall peering is to get the default route
    ! in the evpn-vrf from the internet-vrf. Firewall
    ! does not peer for l2vpn/evpn.
    !
    address-family l2vpn evpn
      advertise ipv4 unicast
!
! internet vrf peering to retrieve the default route from the
! internet facing router and give it to the firewall.
!
router bgp 65041 vrf internet-vrf
    bgp router-id 10.0.0.41
    bgp bestpath as-path multipath-relax
    neighbor internet peer-group
    neighbor internet remote-as external
    neighbor swp1.4 interface peer-group internet
    neighbor swp44 interface peer-group internet
    address-family ipv4 unicast
        neighbor internet activate
        neighbor swp1.4 allowas-in 1
        redistribute connected route-map INTERNET
!
    route-map INTERNET permit 10
        match interface internet-vrf
!



# leaf01âs FRR configuration for symmetric routing looks as follows:

    vrf evpn-vrf
          vni 104001
    !
    router bgp 65011
        bgp router-id 10.0.0.11
        bgp bestpath as-path multipath-relax
        neighbor fabric peer-group
        neighbor fabric remote-as external
        neighbor swp51 interface peer-group fabric
        neighbor swp52 interface peer-group fabric
        address-family ipv4 unicast
            neighbor fabric activate
            redistribute connected route-map LOOPBACKS
        !
        address-family l2vpn evpn
            neighbor fabric activate
            advertise-all-vni
!
    route-map LOOPBACKS permit 10
        match interface lo
!


# A sample Linux interfaces configuration for leaf01 looks as follows:

auto all

iface lo inet loopback
    address 10.0.0.11/32

iface swp51
    mtu 9216

iface swp52
    mtu 9216


# A snippet of Linux interfaces configuration for these two looks as follows:

auto all
iface lo inet loopback
    address 10.0.0.41/32

iface internet-vrf
    vrf-table auto

iface swp44
    vrf internet-vrf


# For everything but Route Type 5 routes (RT-5),
# here is the entire configâ uration for symmetric
# and asymmetric distributed routing configuration
# for leaves, both exit and regular:

  address-family l2vpn evpn
       neighbor fabric activate
       advertise-all-vni


# From our conâ figuration quoted in the previous section, here is that section:

    vrf evpn-vrf
       vni 104001


# In case of centralized routing, the entire EVPN configuration on the exit leaf
# (or any other leaf that performs the task of the centralized router)
# looks as follows:

    address-family l2vpn evpn

       neighbor fabric activate
       advertise-all-vni
       advertise-default-gw


# exit01 in the previous section:

    router bgp 65041 vrf evpn-vrf
        address-family l2vpn evpn
            advertise ipv4 unicast
!

   address-family l2vpn evpn
       neighbor fabric activate


# A snippet of the Linux interfaces configuration for these functions looks like this:

    auto all
    iface lo inet loopback
        address 10.0.0.0.11/32
        clagd-vxlan-anycast-ip 10.0.0.112

    # vlan3 is the SVI for VLAN 3
    iface vlan3
        address 10.1.3.11/24
        address-virtual 44:39:39:ff:00:13 10.1.3.1/24
        vlan-id 3
        vlan-raw-device bridge
        vrf vrf1

    # This is the definition of VNI 3 corresponding to VLAN 3
    iface vni3
      mtu 9000
      vxlan-id 3
      vxlan-local-tunnelip 10.0.0.11
      bridge-access 3
      bridge-learning off

    iface bridge
      bridge-vlan-aware yes

      # bridge-ports includes all ports related to VxLAN and CLAG.
      bridge-ports bond01 bond02 peerlink vni3 vni4
      bridge-vids 3-4



# In case of symmetric routing, creating the VRF and the VLAN/VNI pair looks as follows:

    iface evpn-vrf
       vrf-table auto
    iface vlan4001
      hwaddress 44:39:39:FF:40:94
      vlan-id 4001
      vlan-raw-device bridge
      vrf evpn-vrf

    iface vxlan4001
      vxlan-id 4001
      vxlan-local-tunnelip 10.0.0.11
      bridge-learning off
      bridge-access 4001


# This is the definition of VNI 3 corresponding to VLAN 3

    iface vni3
      mtu 9000
      vxlan-id 3
      vxlan-local-tunnelip 10.0.0.11
      bridge-access 3
      bridge-learning off
      bridge-arp-nd-suppress on



#
#
#

# Example 3-1. Highlighting the router-specific configuration
#  across leaf01 and leaf02

# leaf01âs BGP configuration:

log file /var/log/frr/frr.log
router bgp 65000
      bgp router-id 10.0.254.1
      bgp log-neighbor-changes
      bgp no default ipv4-unicast
      timers bgp 3 9
      neighbor peer-group ISL
      neighbor ISL remote-as 65500
      neighbor ISL advertisement-interval 0
      neighbor ISL timers connect 5
      neighbor 169.254.1.0 peer-group ISL
      neighbor 169.254.1.64 peer-group ISL

      address-family ipv4 unicast
            neighbor ISL activate
            network 10.0.254.1/32
            network 10.1.1.0/26
            maximum-paths 64
      exit-address-family

// leaf02âs BGP configuration

log file /var/log/frr/frr.log
router bgp 65001
      bgp router-id 10.0.254.2
      bgp log-neighbor-changes
      bgp no default ipv4-unicast
      timers bgp 3 9
      neighbor peer-group ISL
      neighbor ISL remote-as 65500
      neighbor ISL advertisement-interval 0
      neighbor ISL timers connect 5
      neighbor 169.254.1.0 peer-group ISL
      neighbor 169.254.1.64 peer-group ISL
      address-family ipv4 unicast
            neighbor ISL activate
            network 10.0.254.1/32
            network 10.1.1.0/26 maximum-paths 64
      exit-address-family




// spine01âs BGP configuration

log file /var/log/frr/frr.log
router bgp 65534
      bgp router-id 10.0.254.254
      bgp log-neighbor-changes
      bgp no default ipv4-unicast
      timers bgp 3 9
      neighbor peer-group ISL
      neighbor ISL advertisement-interval 0
      neighbor ISL timers connect 5
      neighbor 169.254.1.1 remote-as 65000
      neighbor 169.254.1.1 peer-group ISL
      neighbor 169.254.1.3 remote-as 65001
      neighbor 169.254.1.3 peer-group ISL
      neighbor 169.254.1.5 remote-as 65002
      neighbor 169.254.1.5 peer-group ISL
      neighbor 169.254.1.5 remote-as 65003
      neighbor 169.254.1.7 peer-group ISL
      bgp bestpath as-path multipath-relax
      address-family ipv4 unicast
            neighbor ISL activate
            network 10.0.254.254/32 maximum-paths 64
      exit-address-family


// spine02âs BGP configuration

log file /var/log/frr/frr.log
router bgp 65534
      bgp router-id 10.0.254.253
      bgp log-neighbor-changes
      bgp no default ipv4-unicast
      timers bgp 3 9
      neighbor peer-group ISL
      neighbor ISL advertisement-interval 0
      neighbor ISL timers connect 5
      neighbor 169.254.1.1 remote-as 65000
      neighbor 169.254.1.1 peer-group ISL
      neighbor 169.254.1.3 remote-as 65001
      neighbor 169.254.1.3 peer-group ISL
      neighbor 169.254.1.5 remote-as 65002
      neighbor 169.254.1.5 peer-group ISL
      neighbor 169.254.1.5 remote-as 65003
      neighbor 169.254.1.7 peer-group ISL
      bgp bestpath as-path multipath-relax
      address-family ipv4 unicast
            neighbor ISL activate
            network 10.0.254.254/32 maximum-paths 64
      exit-address-family


# The configuration on leaf01 would look like this after replacing
# net work statements with redistribute:

log file /var/log/frr/frr.log
router bgp 65000
      bgp router-id 10.0.254.1
      bgp log-neighbor-changes
      bgp no default ipv4-unicast
      timers bgp 3 9
      neighbor peer-group ISL
      neighbor ISL remote-as 65500
      neighbor ISL advertisement-interval 0
      neighbor ISL timers connect 5
      neighbor 169.254.1.0 peer-group ISL
      neighbor 169.254.1.64 peer-group ISL

      address-family ipv4 unicast
            neighbor ISL activate
            redistribute connected
            maximum-paths 64
      exit-address-family

#

if prefix equals '8.8.8.8/32' then reject else accept


#

if prefix equals 10.1.0.0/16 then accept
    else if prefix equals 10.0.254.1/32 then accept
    else reject

if prefix belongs to 10.1.0.0/16 then accept
    else if prefix belongs to 10.0.254.0/24 then accept
    else reject


if prefix belongs to 10.1.0.0/16 then accept
    else if (prefix belongs to 10.0.254.0/24 and address mask equals 32) then accept
    else reject

ACCEPT_DC_LOCAL(prefix)
    {
        if prefix belongs to 10.1.0.0/16 then accept
        else if (10.0.254.0/24 contains prefix and subnet equals 32) then accept
        else reject
    }


#

# Table 3-1. Key classifiers in the match field of a route-map

as-path                = Match value from BGPâs AS_PATH
community                = Match value from a prefixâs community, if any
extcommunity            = Match value from BGPâs extended community list
interface                = Match name of next hop interface of route
ip, ipv6                = Match IP information such as IP address, nexthop, or source
local-preference        = Match LOCAL_PREFERENCE of route
metric                    = Match routeâs metric field
origin                    = Match routeâs ORIGIN attribute
peer                    = Match session peerâs information


#

ip prefix-list DC_LOCAL_SUBNET seq 5 permit 10.1.0.0/16 le 26
ip prefix-list DC_LOCAL_SUBNET seq 10 permit 10.0.254.0/24 le 32


#

ip prefix-list EXAMPLE_SEQ seq 5 deny 10.1.1.1/32
ip prefix list EXAMPLE_SEQ seq 10 permit 10.1.1.0/24


#

ip prefix-list DC_LOCAL_SUBNET seq 5 permit 10.1.0.0/16 le 26
ip prefix-list DC_LOCAL_SUBNET seq 10 permit 10.0.254.0/24 le 32

route-map ACCEPT_DC_LOCAL permit 10
      match ip-address DC_LOCAL_SUBNET

redistribute connected route-map DC_LOCAL_SUBNET


#

route-map ADV_LO permit 10
      match interface lo

redistribute connected route-map ADV_LO


#

    route-map EXCEPT_ISL_ETH0 deny 10
       match interface swp51

    route-map EXCEPT_ISL_ETH0 deny 20
       match interface swp52

    route-map EXCEPT_ISL_ETH0 deny 30
       match interface eth0

    route-map EXCEPT_ISL_ETH0 permit 40

    redistribute connected route-map EXCEPT_ISL_ETH0


#

neighbor 169.254.1.1 route-map NBR_RT_ACCEPT in
neighbor 169.254.1.1 route-map NBR_RT_ADV out

network 10.1.1.1/24 route-map ADV_NET

neighbor foo default-originate route-map ONLY_NON_EXITS



#

neighbor 169.254.1.0 peer-group ISL
neighbor 169.254.1.64 peer-group ISL

neighbor swp51 interface peer-group ISL
neighbor swp52 interface peer-group ISL


#

// leaf01âs BGP configuration

log file /var/log/frr/frr.log

ip prefix-list DC_LOCAL_SUBNET 5 permit 10.1.0.0/16 le 26
ip prefix-list DC_LOCAL_SUBNET 10 permit 10.0.254.0/24 le 32

route-map ACCEPT_DC_LOCAL permit 10
     match ip-address DC_LOCAL_SUBNET

router bgp 65000
     bgp router-id 10.0.254.1
     bgp log-neighbor-changes
     bgp no default ipv4-unicast
     timers bgp 3 9

     neighbor peer-group ISL
     neighbor ISL remote-as 65500
     neighbor ISL advertisement-interval 0
     neighbor ISL timers connect 5
     neighbor swp51 peer-group ISL
     neighbor swp52 peer-group ISL

     address-family ipv4 unicast
          neighbor ISL activate
          redistribute connected route-map DC_LOCAL
          maximum-paths 64

     exit-address-family



#

// leaf02âs BGP configuration

log file /var/log/frr/frr.log
ip prefix-list DC_LOCAL_SUBNET 5 permit 10.1.0.0/16 le 26
ip prefix-list DC_LOCAL_SUBNET 10 permit 10.0.254.0/24 le 32

route-map ACCEPT_DC_LOCAL permit 10
     match ip-address DC_LOCAL_SUBNET

router bgp 65001
     bgp router-id 10.0.254.2
     bgp log-neighbor-changes
     bgp no default ipv4-unicast
     timers bgp 3 9

     neighbor peer-group ISL
     neighbor ISL remote-as 65500
     neighbor ISL advertisement-interval 0
     neighbor ISL timers connect 5
     neighbor swp51 peer-group ISL
     neighbor swp52 peer-group ISL

     address-family ipv4 unicast
          neighbor ISL activate
          redistribute connected route-map DC_LOCAL
          maximum-paths 64
     exit-address-family



log file /var/log/frr/frr.log

ip prefix-list ACCRT 5 permit 10.1.0.0/16 le 26
ip prefix-list ACCRT 10 permit 10.0.254.0/24 le 32

route-map DC_LOCAL permit 10
     match ip-address ACCRT

router bgp 65500
     bgp router-id 10.0.254.254
     bgp log-neighbor-changes
     bgp no default ipv4-unicast
     timers bgp 3 9

     neighbor peer-group ISL
     neighbor ISL advertisement-interval 0
     neighbor ISL timers connect 5
     neighbor swp1 remote-as 65000
     neighbor swp1 peer-group ISL
     neighbor swp2 remote-as 65001
     neighbor swp2 peer-group ISL
     neighbor swp3 remote-as 65002
     neighbor swp3 peer-group ISL
     neighbor swp4 remote-as 65003
     neighbor swp4 peer-group ISL

     bgp bestpath as-path multipath-relax

     address-family ipv4 unicast
          neighbor ISL activate
          redistribute connected route-map DC_LOCAL
          maximum-paths 64

     exit-address-family



#

    ROUTE: 10.1.1.0/24 via 169.254.0.1 dev swp1
    ARP: 169.254.0.1 dev swp1 lladdr 00:00:01:02:03:04 PERMANENT

#

    BGP unnumbered uses the interfaceâs IPv6 LLA to set up a BGP session with a peer.

    The IPv6 LLA of the remote end is discovered via IPv6âs Router Advertisement (RA) protocol.

    RA provides not only the remote endâs LLA, but also its correâ sponding MAC address.

    BGP uses RFC 5549 to encode IPv4 routes as reachable over an IPv6 nexthop, using the IPv6 LLA as the nexthop.

    The RIB process programs a static ARP entry with a reserved IPv4 LLA, 169.254.0.1, with the MAC address set to the one learned via RA.

    BGP hands down to the RIB process IPv4 routes with the IPv6 LLA as the nexthop.

    The RIB process converts the nexthop to 169.254.0.1 and the outgoing interface before programming the route in the forâ warding table.

    BGP Capability to Negotiate RFC 5549 Use


#

// leaf01 configuration

log file /var/log/frr/frr.log

ip prefix-list DC_LOCAL_SUBNET 5 permit 10.1.0.0/16 le 26
ip prefix-list DC_LOCAL_SUBNET 10 permit 10.0.254.0/24 le 32

route-map ACCEPT_DC_LOCAL permit 10
  match ip-address DC_LOCAL_SUBNET

router bgp 65000
      bgp router-id 10.0.254.1
      neighbor peer-group ISL
      neighbor ISL remote-as external
      neighbor swp51 interface peer-group ISL
      neighbor swp52 interface peer-group ISL
      address-family ipv4 unicast
            neighbor ISL activate
            redistribute connected route-map ACCEPT_DC_LOCAL


// spine01 configuration

log file /var/log/frr/frr.log

ip prefix-list DC_LOCAL_SUBNET 5 permit 10.1.0.0/16 le 26
ip prefix-list DC_LOCAL_SUBNET 10 permit 10.0.254.0/24 le 32

route-map ACCEPT_DC_LOCAL permit 10
      match ip-address DC_LOCAL_SUBNET

router bgp 65534
      bgp router-id 10.0.254.254
      neighbor peer-group ISL
      neighbor ISL remote-as external
      neighbor swp1 interface peer-group ISL
      neighbor swp2 interface peer-group ISL
      neighbor swp3 interface peer-group ISL
      neighbor swp4 interface peer-group ISL
      address-family ipv4 unicast
            neighbor ISL activate
            redistribute connected route-map ACCEPT_DC_LOCAL


#


route-map SCHED_MAINT permit 10
      set as-path prepend 65000 65000

neighbor ISL route-map SCHED_MAINT out


#



#




#




#




#



#
#
#



#
#
#






#
#
#






#
#
#






#
#
#






#
#
#






#
#
#






#
#
#






#
#
#






#
#
#






#
#
#






#
#
#






#
#
#






#
#
#






#
#
#






#
#
#



























#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-  peering com KENTIK (FAKE POC)


config t

    # Rota IGP (static) (necessario para iBGP)
    ip route 172.233.24.85/32 10.255.206.1 1

    router bgp 262493
        no neighbor 172.233.24.85

    router bgp 262493
        neighbor 172.233.24.85 remote-as internal
        neighbor 172.233.24.85 advertisement-interval 0
        neighbor 172.233.24.85 timers connect 1
        neighbor 172.233.24.85 ttl-security hops 250
        neighbor 172.233.24.85 solo
        neighbor 172.233.24.85 update-source 191.37.79.225
        neighbor 172.233.24.85 tcp-mss 1200

        address-family ipv4 unicast
            neighbor 172.233.24.85 activate
            neighbor 172.233.24.85 allowas-in 1
            neighbor 172.233.24.85 route-reflector-client
            neighbor 172.233.24.85 soft-reconfiguration inbound
            neighbor 172.233.24.85 send-community all
            neighbor 172.233.24.85 route-map ALLOW-ALL in
            neighbor 172.233.24.85 route-map ALLOW-ALL out

end
write











#====================================================================================
#====================================================================================
#====================================================================================
#====================================================================================
#====================================================================================













