#!/bin/sh


# Montar servidor dns recursivo com unbound em servidor Debian (11 ou 12)


# Colocar IPs ANYCAST nas loopbacks
    _interface_register(){
        interface_file="/etc/network/interfaces"
        interface_name="$1"
        interface_addr="$2"
        addr_proto=inet
        echo "$interface_addr" | egrep -q ':' && addr_proto=inet6
        interface_addr_regex=$(echo "$interface_addr" | sed 's#\.#\\.#g')
        egrep -q "$interface_addr_regex" "$interface_file" || {
            # requer registro
            #echo "Requer registro de [$interface_addr] em [$interface_name] no arquivo [$interface_file]"
            (
                echo ''
                echo "auto $interface_name"
                echo "iface $interface_name  $addr_proto static"
                echo "    address $interface_addr"
                echo ''
            ) >> $interface_file
            ifup --force "$interface_name" 2>/dev/null
        }
    }
    # Loopbacks de DNS
    _interface_register lo:1  10.255.255.254/32
    _interface_register lo:2  2001:db8:ffff:ffff::254/128



# Baixar lista de root-servers
    # /etc/unbound/named.root
    mkdir -p /etc/unbound
    wget --no-check-certificate -O /etc/unbound/named.root https://www.internic.net/domain/named.root




# Ativar unbound (requer config)
    _enable_service unbound


# -- Montar unbound recursivo loopback anycast de dns publico
    mkdir -p /etc/unbound/unbound.conf.d

    # remover config padrao e anterior (cuidado, tenha backup)
    rm -f /etc/unbound/unbound.conf.d/* 2>/dev/null

    # config baseada em include.d
    (
        echo 'include-toplevel: "/etc/unbound/unbound.conf.d/*.conf"'
    ) > /etc/unbound/unbound.conf


    # server default
    (
        echo ''
        echo 'server:'
        echo '    auto-trust-anchor-file: "/var/lib/unbound/root.key"'
        echo '    verbosity: 1'
        echo '    statistics-interval: 30'
        echo '    extended-statistics: yes'
        echo '    minimal-responses: yes'
        echo '    #prefetch: yes'
        echo '    #prefetch-key: yes'
        echo '    #serve-expired: yes'
        echo ''
        echo '    identity: "xProxy-DNS"'
        echo '    version: "1.0"'
        echo '    hide-identity: yes'
        echo '    hide-version: yes'
        echo ''
        echo '    harden-glue: yes'
        echo '    do-not-query-address: 127.0.0.1/8'
        echo '    do-not-query-localhost: yes'
        echo ''
        echo '    username: "unbound"'
        echo '    directory: "/etc/unbound"'
        echo '    pidfile: "/run/unbound.pid"'
        echo ''
        echo '    root-hints: "/etc/unbound/named.root"'
        echo ''
    ) > /etc/unbound/unbound.conf.d/001-server-config.conf

    # threads
    (
        echo ''
        echo 'server:'
        echo '    num-threads: 16'
        echo '    outgoing-range: 8192'
        echo '    num-queries-per-thread: 4096'
        echo ''
    ) > /etc/unbound/unbound.conf.d/002-server-threads.conf

    # listen
    (
        echo ''
        echo 'server:'
        echo '    interface: 127.0.0.1'
        echo '    interface: ::1'
        echo '    interface: 8.8.8.8'
        echo '    interface: 4.2.2.2'
        echo '    interface: 10.255.255.254'
        echo ''
    ) > /etc/unbound/unbound.conf.d/003-server-listen.conf

    # ip de saida
    # caso seja necessario fixar o ip de origem:
    (
        src_ipv4=$(ip -o -4 ro get 1.2.3.4      2>/dev/null | sed 's#.*src.##g' | cut -f1 -d' ')
        src_ipv6=$(ip -o -6 ro get 2001:abcd::1 2>/dev/null | sed 's#.*src.##g' | cut -f1 -d' ')
        echo ''
        echo 'server:'
        [ "x$src_ipv4" = "x" ] || echo "    outgoing-interface: $src_ipv4"
        [ "x$src_ipv6" = "x" ] || echo "    outgoing-interface: $src_ipv6"
        echo ''
    ) > /etc/unbound/unbound.conf.d/004-server-outgoing.conf


    # transport over IP
    (
        echo ''
        echo 'server:'
        echo '    do-ip4: yes'
        echo '    do-ip6: yes'
        echo '    do-udp: yes'
        echo '    do-tcp: yes'
        echo '    do-daemonize: yes'
        echo ''
        echo '    so-reuseport: yes'
        echo '    so-rcvbuf: 4m'
        echo '    so-sndbuf: 4m'
        echo ''
    ) > /etc/unbound/unbound.conf.d/005-server-transport.conf

    # daemon
    (
        echo ''
        echo 'server:'
        echo '    #do-daemonize: yes'
        echo '    chroot: ""'
        echo ''
    ) > /etc/unbound/unbound.conf.d/006-server-daemonize.conf


    # cache
    (
        echo ''
        echo 'server:'
        echo '    rrset-cache-size: 256m'
        echo '    rrset-cache-slabs: 4'
        echo '    rrset-roundrobin: yes'
        echo '    msg-cache-size: 512m'
        echo '    msg-cache-slabs: 4'
        echo '    cache-max-ttl: 1200'
        echo '    infra-host-ttl: 60'
        echo '    infra-lame-ttl: 60'
        echo '    infra-cache-numhosts: 128'
        echo ''
        echo '    edns-buffer-size: 1232'
        echo ''
    ) > /etc/unbound/unbound.conf.d/007-server-cache.conf
    

    # log
    (
        echo ''
        echo 'server:'
        echo '    #log-time-ascii: yes'
        echo '    #logfile: "/var/log/unbound.log"'
        echo '    use-syslog: yes'
        echo '    log-queries: no'
        echo ''
    ) > /etc/unbound/unbound.conf.d/008-server-log.conf
    


    # access control
    (
        echo ''
        echo 'server:'
        echo '    # Permitir todos os IPs privados'
        echo '    access-control: 192.168.0.0/16 allow'
        echo '    access-control: 172.16.0.0/12 allow'
        echo '    access-control: 100.64.0.0/10 allow'
        echo '    access-control: 10.0.0.0/8 allow'
        echo '    access-control: 127.0.0.0/8 allow'
        echo '    access-control: ::1/128 allow'
        echo ''
        echo '    # Permitir todos os IPs pois abrimos a porta'
        echo '    # apenas para a loopback'
        echo '    access-control: 0.0.0.0/0 allow'
        echo '    access-control: ::/0 allow'
        echo ''
    ) > /etc/unbound/unbound.conf.d/021-server-access-control.conf


    # interator - domains
    (
        echo ''
        echo 'server:'
        echo '    module-config: "iterator"'
        echo ''
        echo '    local-zone: "localhost." static'
        echo '    local-data: "localhost. 10800 IN NS localhost."'
        echo '    local-data: "localhost. 10800 IN SOA localhost. nobody.invalid. 1 3600 1200 604800 10800"'
        echo '    local-data: "localhost. 10800 IN A 127.0.0.1"'
        echo ''
        echo '    local-zone: "intranet.br." static'
        echo '    local-data: "intranet.br. 10800 IN SOA ns.intranet.br. patrickbrandao.intranet.br. 2023113015 3600 1200 604800 10800"'
        echo '    local-data: "intranet.br. 10800 IN NS ns.intranet.br."'
        echo '    local-data: "ns.intranet.br. 10800 IN A 10.255.255.254"'
        echo '    local-data: "intranet.br. 10800 IN A 10.255.255.254"'
        echo '    local-data: "gateway.intranet.br. 10800 IN A 172.19.255.254"'
        for n in $(seq 0 1 63); do
            NID=$n
            [ "$n" -lt "10" ] && NID="0$n"
            echo "    local-data: \"anablock-$NID.intranet.br. 10800 IN A 172.19.1.$n\""
        done
        echo ''
        echo '    local-zone: "127.in-addr.arpa." static'
        echo '    local-data: "127.in-addr.arpa. 10800 IN NS localhost."'
        echo '    local-data: "127.in-addr.arpa. 10800 IN SOA localhost. nobody.invalid. 2 3600 1200 604800 10800"'
        echo '    local-data: "1.0.0.127.in-addr.arpa. 10800 IN PTR localhost."'
        echo ''
        echo ''
    ) > /etc/unbound/unbound.conf.d/101-module-iterator.conf


    # reversos de ips privados
    (
        echo ''
        echo 'server:'
        echo '    local-zone: "10.in-addr.arpa."      static'
        echo '    local-zone: "16.172.in-addr.arpa."  static'
        echo '    local-zone: "17.172.in-addr.arpa."  static'
        echo '    local-zone: "18.172.in-addr.arpa."  static'
        echo ''
        echo '    local-zone: "19.172.in-addr.arpa." static'
        echo '    local-data: "19.172.in-addr.arpa. 10800 IN SOA ns.intranet.br. patrickbrandao.intranet.br. 2023113015 3600 1200 604800 10800"'
        echo '    local-data: "19.172.in-addr.arpa. 10800 IN NS ns.intranet.br."'
        echo '    local-data: "254.255.19.172.in-addr.arpa. 10800 IN PTR gateway.intranet.br."'
        for n in $(seq 0 1 63); do
            NID=$n
            [ "$n" -lt "10" ] && NID="0$n"
            echo "    local-data: \"$n.1.19.172.in-addr.arpa. 10800 IN PTR anablock-$NID.intranet.br.\""
        done
        echo ''
        echo '    local-zone: "20.172.in-addr.arpa."  static'
        echo '    local-zone: "21.172.in-addr.arpa."  static'
        echo '    local-zone: "22.172.in-addr.arpa."  static'
        echo '    local-zone: "23.172.in-addr.arpa."  static'
        echo '    local-zone: "24.172.in-addr.arpa."  static'
        echo '    local-zone: "25.172.in-addr.arpa."  static'
        echo '    local-zone: "26.172.in-addr.arpa."  static'
        echo '    local-zone: "27.172.in-addr.arpa."  static'
        echo '    local-zone: "28.172.in-addr.arpa."  static'
        echo '    local-zone: "29.172.in-addr.arpa."  static'
        echo '    local-zone: "30.172.in-addr.arpa."  static'
        echo '    local-zone: "31.172.in-addr.arpa."  static'
        echo '    local-zone: "168.192.in-addr.arpa." static'
        echo ''
    ) > /etc/unbound/unbound.conf.d/102-private-zone.conf


    # ** declarar dominios aqui

    # modo forward (desativado por padrao)
    (
        echo ''
        echo '    # Operar 100% em modo forward, informe o ip dos servidores DNSs reais:'
        echo '    #forward-zone:'
        echo '    #    name: "."'
        echo '    #    forward-addr: 2001:4860:4860::8888'
        echo '    #    forward-addr: 8.8.8.8'
        echo '    #    forward-addr: 8.8.4.4'
        # echo '    forward-zone:'
        # echo '        name: "."'
        # echo '        forward-addr: 2001:4860:4860::8888'
        # echo '        forward-addr: 8.8.8.8'
        # echo '        forward-addr: 8.8.4.4'
        # echo ''
        echo '    # Modo forward de dominio especifico:'
        echo '    #forward-zone:'
        echo '    #    name: "slack.com"'
        echo '    #    forward-addr: 1.1.1.1'
        echo '    #    forward-addr: 1.0.0.1'
        echo ''
    ) > /etc/unbound/unbound.conf.d/992-forward-zone.conf


    # controle remoto e estatisticas
    (
        echo ''
        echo 'remote-control:'
        echo '    control-enable: no'
        echo '    #control-interface: /run/unbound.ctl'
        echo '    #control-interface: 127.0.0.1'
        echo '    #control-port: 8953'
        echo '    #control-use-cert: "no"'
        echo ''
    ) > /etc/unbound/unbound.conf.d/998-remote-control.conf

  


# LOGROTATE
    (
        echo '/var/log/unbound/unbound.log {'
        echo '    rotate 5'
        echo '    weekly'
        echo '    postrotate'
        echo '        unbound-control log_reopen'
        echo '    endscript'
        echo '}'
    ) > /etc/logrotate.d/unbound
    systemctl restart logrotate.service

    touch /var/log/unbound.log
    chown unbound:unbound /var/log/unbound.log

# Homologar internet
    wget tmsoft.com.br/temp/internet-test.sh -O /usr/bin/internet-test
    chmod +x /usr/bin/internet-test

    internet-test







