<?php

	// funções para trabalhar com IPv4

	// conversao de formato ip2int (grandeza decimal)
	function &ip2int(&$_ip){
		$_a = explode('.', $_ip);
		$_n = 0;
		foreach($_a as $_k=>$_v){
			if(!$_k) continue;
			$_a[$_k] = substr("000".$_v, -3);
		}
		$_n = (implode('', $_a))*1;
		return($_n);
	}

	// conversao de formato mac2int
	function &int2ip(&$_int){
		$_ip = array(0,0,0,0);
		$_str = (string)$_int;
		$_str = substr(str_repeat("0", 12).$_str, -12);
		$_k = 0;
		for($_i=0;$_i<12;$_i+=3){
			$_ip[$_k++] = (int)substr($_str, $_i, 3);
		}
		$_ip = implode('.', $_ip);
		return($_ip);
	}

	// carrega informacoes sobre endereço e mascara
	function &ipv4_load($arg_ip, $arg_bitmask = 33){
		$_ip = $arg_ip;
		$_bitmask = $arg_bitmask;

		// padrao da resposta
		$r = array(
			'address' => '',	// endereco passado em $_ip
			'prefix' => '',		// endereco de rede e bits da mascara
			'prefixlen' => 0,	// numeros de bits na mascara
			'bitmask' => 0,		// mascara numerica
			'mask' => '',		// mascara string ip
			'type' => 0,		// 0=host, 1=net, 2=brd
			'network' => '',	// ip da rede
			'broadcast' => '',	// ip de broadcast
			'count' => 0,		// numero de hosts (1 auando type=0, ou quantidade de ips na rede)
			'private' => 0,		// ip privado ou publico
			'first' => '',		// primeiro ip da rede (caso type=1 ou 2)
			'last' => '',		// ultimo ip da rede,
			'class' => '',		// classe A, B, C, L, M
			'ipv' => 4			// versao do protocolo ip
		);
		// carregar ip enviado
		$_ip = trim($_ip);

		// detectar presenca de mascara
		//echo "IN-IP: $_ip\n";
		$_bp = strpos($_ip, '/');
		if($_bp!==false){
			$_bitmask = (int)substr($_ip, $_bp+1);
			$_ip = substr($_ip, 0, $_bp);
		}
		if($_bitmask > 32) $_bitmask = 32;

		//echo "IN-BITMASK: $_bitmask\n";
		// ler cada byte numerico, ajuda a remover zeros a esquerda
		$a = explode('.', $_ip);
		foreach($a as $k=>$v) $a[$k] = (int)$v;
		$_ip = implode('.', $a); unset($a);
		$r['address'] = $_ip;

		// processar mascara
		if(is_int($_bitmask)||is_numeric($_bitmask)){
			$_bitmask = (int)$_bitmask;
			//echo "IN-BITMASK-A-1: $_bitmask\n";
			if($_bitmask > 32) $_bitmask = 32;
			if($_bitmask < 0) $_bitmask = 0;
			//echo "IN-BITMASK-A-2: $_bitmask\n";
		}else{
			// string
			$_bitmask = ipv4_mask2bitmask($_bitmask);
			//echo "IN-BITMASK-B: $_bitmask\n";
		}
		$r['bitmask'] = $_bitmask;
		$r['prefixlen'] = $_bitmask;

		// obter mascara string
		$r['mask'] = ipv4_bitmask2mask($_bitmask);
		

		// Gerar binario
		$r['fullbin'] = ipv4_ip2bin($_ip);
		$r['netbits'] = substr($r['fullbin'], 0, $_bitmask);
		$r['hostbits'] = substr($r['fullbin'], -1 * (32 - $_bitmask ));

		// calcular endereço de rede e broadcast
		$_ipm = ipv4_mask($_ip, $_bitmask);
		foreach($_ipm as $_k=>$_v) $r[$_k]=$_v;
	
		// ajustar tipo
		if($r['address'] == $r['network']) $r['type'] = 1;
		if($r['address'] == $r['broadcast']) $r['type'] = 2;

		// verificar se é um ip privado
		$r['private'] = ipv4_isprivate($r['address']);

		// Prefixo
		$r['prefix'] = $r['network'] . '/' . $r['prefixlen'];

		// classe
		$p = explode('.', $r['address']);

		// ($_ip, $_bitmask = 33)
		// if(!isset($p[1])){
		// 	echo "ipv4: falha de execucao, argumentos:\n";
		// 	echo "   arg1: $arg_ip\n";
		// 	echo "   arg2: $arg_bitmask\n";
		// 	echo "\n";
		// 	echo "P array: "; print_r($p);
		// 	echo "\n";
		// 	exit();
		// }

		$p[0] = (int)$p[0];
		$p[1] = (int)$p[1];


		$r['class'] = 'Z';
		if($p[0] && $p[0] <= 126){
			$r['class'] = 'A';

		}elseif($p[0] == 127){
			$r['class'] = 'L';

		}elseif($p[0] >= 128 && $p[0] <= 191){
			$r['class'] = 'B';

		}elseif($p[0] >= 192 && $p[0] <= 223){
			$r['class'] = 'C';

		}elseif($p[0] >= 224 && $p[0] <= 239){
			$r['class'] = 'M';

		}else{
			$r['class'] = 'E';
		}

		/*

		A		1.0.0.0 até 126.0.0.0		16 777 216
		B		128.0.0.0 até 191.255.0.0		65 536
		C		192.0.0.0 até 223.255.255.0		256
		D		224.0.0.0 até 239.255.255.255		Multicast
		E		240.0.0.0 até 255.255.255.254		Uso futuro; atualmente reservada a testes pela IETF

		*/

		return($r);
	}


	// verificar se ip é valido sintaticamente para comunicação como destinatario
   function ipv4_test_dst($_ip){
        $_ip = trim($_ip);  
        $_len = strlen($_ip);
        // testar tamanho  
        if($_len < 7 || $_len > 15) return false;  
        // testar partes, precista ter 3 pontos  
        if(substr_count($_ip, '.')!=3) return false;  
        $_parts = explode('.', $_ip);  
        foreach($_parts as $k=>$v){  
            $v = (int)$v;  
            if($v<0||$v>255) return false;  
            $_parts[$k] = $v;  
        }  
        // primeira parte nao pode ser zero nem 255, nem acima da faixa C maxima 238  
        if($_parts[0]<1||$_parts[0]>=239) return false;
        return(true);
   }

   function ipv4_test($_ip){
        $_ip = trim($_ip);  
        $_len = strlen($_ip);  
        // testar tamanho  
        if($_len < 7 || $_len > 15) return false;  
        // testar partes, precista ter 3 pontos  
        if(substr_count($_ip, '.')!=3) return false;  
        $_parts = explode('.', $_ip);
        foreach($_parts as $k=>$v){  
            $v = (int)$v;  
            if($v<0||$v>255) return false;  
            $_parts[$k] = $v;  
        }  
        return true;
   }

	// converter mascara string IP em mascara decimal (numero de bits ligados)
	function ipv4_mask2bitmask($_mask){
		if(!ipv4_test($_mask)) return(32);
		// é sintaticamente correta, partir
		$_a = explode('.', $_mask);
		$_n = 0;
		foreach($_a as $_pn){
			$_bin = base_convert($_pn, 10, 2);
			$_bin = substr('00000000'.$_bin,-8);
			$_c1 = substr_count($_bin, '1');
			$_c0 = substr_count($_bin, '0');
			if(!$_c1) break;
			if($_c0){
				// contar bits em 1 no inicio
				$_zp = strpos($_bin, '0');
				if($_zp>0) $_n+=strlen(substr($_bin, 0, $_zp));
				break;
			}
			$_n+=$_c1;
		}
		return($_n);
	}

	// converter mascara string IP em mascara decimal (numero de bits ligados)
	function ipv4_bitmask2mask($_bitsnum){
		$_bitsnum = (int)$_bitsnum;
		if($_bitsnum<0||$_bitsnum>32) $_bitsnum = 32;

		$_ip = array('0', '0', '0', '0');
		$_k = 0;
		while($_bitsnum){
			if($_bitsnum<=8){
				switch($_bitsnum){
					case 8: $_ip[$_k]='255';break;
					case 7: $_ip[$_k]='254';break;
					case 6: $_ip[$_k]='252';break;
					case 5: $_ip[$_k]='248';break;
					case 4: $_ip[$_k]='240';break;
					case 3: $_ip[$_k]='224';break;
					case 2: $_ip[$_k]='192';break;
					case 1: $_ip[$_k]='128';break;
				}
				break;
			}else{
				$_ip[$_k]='255';
			}
			$_k++;
			$_bitsnum-=8;
		}
		return(implode('.', $_ip));
	}

	// converter ip em binario (baseado em partes)
	function ipv4_ip2bin($_ip){
		// obter partes do ip
		$_ipt = explode('.', $_ip); $_ip = '';
		foreach($_ipt as $_p) $_ip.=substr('00000000'.base_convert($_p, 10, 2), -8);
		$_l = strlen($_ip);
		if($_l!=32) $_ip = substr(str_repeat('0', 32).$_ip, -32);
		return($_ip);
	}
	// converter binario em ip (baseado em partes)
	function ipv4_bin2ip($_bin){
		$_l = strlen($_bin);
		if($_l != 32) $_bin = substr(str_repeat('0', 32).$_bin, -32);
		$_k = 0;
		$_ip = array();
		for($_i=0;$_i<=24;$_i+=8){
			$_p = substr($_bin, $_i, 8);
			$_ip[$_k] = base_convert($_p, 2, 10);
			$_k++;
		}
		return(implode('.', $_ip));
	}


	// função semelhante ao comando ipmask, recebe ip e mascara, retorna network e broadcast
	function &ipv4_mask($_ip, $_net){
		if(!is_int($_net)) $_net = ipv4_mask2bitmask($_net);
		
		// obter partes do ip
		$_ipb = ipv4_ip2bin($_ip);
		
		// obter binario da mascara de rede
		//$_binmask = str_repeat('1', $_net).str_repeat('0', 32-$_net);
		$_hosts_bits = 32 - $_net;
	
		// obter parte protejida pelas mascara
		$_net_prefix_bin = substr($_ipb, 0, $_net);
		$_net_address = ipv4_bin2ip( $_net_prefix_bin . str_repeat('0', 32 - $_net) );
		$_brd_address = ipv4_bin2ip( $_net_prefix_bin . str_repeat('1', 32 - $_net) );

		// endereços, primeiro e ultimo
		$_first = $_ip;
		$_last = $_ip;
		if($_hosts_bits>1){
			$_first = ipv4_bin2ip($_net_prefix_bin.str_repeat('0', 32-$_net-1).'1');
			$_last = ipv4_bin2ip($_net_prefix_bin.str_repeat('1', 32-$_net-1).'0');
		}elseif($_hosts_bits==1){
			$_first = ipv4_bin2ip($_net_prefix_bin.str_repeat('0', 32-$_net-1).'1');
			$_last = ipv4_bin2ip($_net_prefix_bin.str_repeat('1', 32-$_net-1).'0');
		}

		// calculcular numero de hosts, 1 2 4 8 16
		$_bh = 32 - $_net;
		$_c = 0;
		if($_bh > 0) $_c = base_convert(str_repeat("1", $_bh), 2,10);
		$_c = (int)$_c;
		$_c-=2;	// subtrarir endereço de rede e broadcast da contagem
		if($_c < 1)$_c=1;

		// Numero de hosts baseado nos bits de host
		$bcount = 1;
		if($_hosts_bits){
			// calcular decimal pelo numero de bits em ON
			$i = $_hosts_bits; while($i > 0){ $bcount *= 2; $i--; }
		}

		// retorno
		$r = array(
			'address' => $_ip,
			'network' => $_net_address,
			'broadcast' => $_brd_address,
			'first' => $_first,
			'last' => $_last,
			'count' => $_c,
			'bcount' => $bcount
		);
		return($r);
	}

	// verificar se um endereço IP é privado (RFC 1918 ou ZeroConf)
	function ipv4_isprivate($_ip){
		$_a = explode('.', $_ip);
		if(count($_a)!=4) return 0;
		foreach($_a as $_k=>$_v) $_a[$_k] = (int)$_v;
		if($_a[0]===10) return 1;
		if($_a[0]===192&&($_a[1]===168||$_a[1]===0)) return 1;
		if($_a[0]===169&&$_a[1]===254) return 1;
		if($_a[0]===198&&($_a[1]===18||$_a[1]===19)) return 1;
		if($_a[0]===172&&($_a[1]>=16&&$_a[1]<=31)) return 1;
		if($_a[0]===100&&($_a[1]>=64&&$_a[1]<=127)) return 1;
		if($_a[0]===223&&$_a[1]===255&&$_a[2]==255) return 1;
		return 0;
	}

	// ler ip e retornar interpretacao simples
	function ipv4_read($_ipstr){
		$_ipstr = trim($_ipstr);
		$_ip = '';
		$_mask = '';
		$_l = strlen($_ipstr);
		$_fm = 0;
		$_last = '';
		for($i=0;$i<$_l;$i++){
			$_c = substr($_ipstr, $i,1);
			if($_c=='/'){ $_fm=1; continue; }
			$_in = ($_c=='0'||$_c=='1'||$_c=='2'||$_c=='3'||$_c=='4'||$_c=='5'||$_c=='6'||$_c=='7'||$_c=='8'||$_c=='9')?true:false;
			if($_fm && $_in) $_mask.=$_c;
			if(!$_fm && ($_in||$_c=='.')) $_ip.=($_c=='.'&&$_last=='.'?'?.': $_c);
			$_last = $_c;
		}
		if(!substr_count($_ip, '.')) $_ip.='.';
		if(substr($_ip, -1)=='.') $_ip.='?';
		
		// preencher casas vazias do IP
		$_a = array();
		if(strpos($_ip, '.')!==false) $_a = explode('.', $_ip);
		for($i=0;$i<4;$i++) if(!isset($_a[$i])) $_a[$i] = '?';
		$_ip = implode('.', $_a);

		if($_mask==''){
			// determinar mascara pelo numero de '?'
			$_mask = (4-substr_count($_ip, '?'))*8;
			if(!$_mask) $_mask = '32';
		}
		$_ip = str_replace('?', '0', $_ip);

		//echo "	IP[$_ip] MASK[$_mask]\n";

		// interpretar mascara
		if(strlen($_mask) < 3){
			// mascara numerica
			$_mask = (int)$_mask;
			$_mask = ipv4_bitmask2mask($_mask);
		}else{
			if(!ipv4_test($_mask)) $_mask = '255.255.255.255';
		}

		// carregar dados do ip
		$_x = ipv4_load($_ip, $_mask);
		$_ret = $_x['address'];
		if($_x['type']===1){
			// endereco de rede
			$_ret = $_x['address'].'/'.$_x['bitmask'];
		}
		return $_ret;
	}

	// Verificar se duas redes colidem ou fazem overlap
	function ipv4_net_overlap($network_a, $network_b){
		// Carregar primeira rede
		$net_a = ipv4_load($network_a);
		$net_a_bits = substr(ipv4_ip2bin($net_a['network']), 0, $net_a['bitmask'] );
		// Carregar segunda rede
		$net_b = ipv4_load($network_b);
		$net_b_bits = substr(ipv4_ip2bin($net_b['network']), 0, $net_b['bitmask'] );
		// Verificar se as redes participam um do outro
		$p = false;
		if($net_a['bitmask']>=$net_b['bitmask'])$p=(substr($net_a_bits,0,$net_b['bitmask'])==substr($net_b_bits,0,$net_b['bitmask']));
		if($net_a['bitmask']<=$net_b['bitmask'])$p=(substr($net_a_bits,0,$net_a['bitmask'])==substr($net_b_bits,0,$net_a['bitmask']));
		return $p===false ? 0 : 1;
	}


	// Verificar se o IP e' multicast
	// 224.0.0.0 through 239.255.255.255
	function ipv4_is_multicast($mipv4addr){
		if(!ipv4_test($mipv4addr)) return 0;
		$n = ip2long($mipv4addr);
		$ini = ip2long('224.0.0.0');
		$end = ip2long('239.255.255.255');
		if($n >= $ini && $n <= $end) return 1;
		return 0;
	}


	// Quebrar prefixo IP em pedacos
	// Argumentos:
	// - prefix......: prefixo (x.x.x.x/nn)
	// - prefix_len..: tamanho do prefixo das partes geradas
	// - inc_plen....: incluir mascara nos sub-prefix
	// - limit.......: limitar numero de prefixos, 0=nao limitar
	//
	function &ipv4_get_subnets($prefix, $subnet_prefix_len=27, $inc_plen=1, $limit=0){
		$list = array();

		$ipinfo = ipv4_load($prefix);

		$pfxsuffix = ($inc_plen ? '/' . $subnet_prefix_len : '');

		// echo "ipv4_load info for $prefix: \n"; print_r($ipinfo); // exit();
		/*
			ipv4_load info for 192.0.0.1/32: 
			Array
			(
			    [address] => 192.0.0.1
			    [prefix] => 192.0.0.1/32
			    [prefixlen] => 32
			    [bitmask] => 32
			    [mask] => 255.255.255.255
			    [type] => 2
			    [network] => 192.0.0.1
			    [broadcast] => 192.0.0.1
			    [count] => 1
			    [private] => 1
			    [first] => 192.0.0.1
			    [last] => 192.0.0.1
			    [class] => C
			    [fullbin] => 11000000000000000000000000000001
			    [netbits] => 11000000000000000000000000000001
			    [hostbits] => 11000000000000000000000000000001
			    [bcount] => 1
			)
		*/

		// print_r($ipinfo);

		// Prefixo do tamanho certo
		if($ipinfo['prefixlen']==$subnet_prefix_len){
			$list[] = $ipinfo['network'] . $pfxsuffix;
			// $list[] = $prefix;
			return $list;
		}

		// Prefixo menor que o pedaco solicitado
		if($ipinfo['prefixlen'] > $subnet_prefix_len) return $list;

		// Prefixo maior, calcular numero de pedacos possiveis
		$diffbits = $subnet_prefix_len - $ipinfo['prefixlen'];
		$subcount = pow(2, $diffbits);

		// echo "#debug> diffbits=$diffbits subcount=$subcount\n";

		// Gerar subnets
		$netbits = $ipinfo['netbits'];
		$zeros = str_repeat('0', 32);
		$c = 0;
		for($n = 0; $n < $subcount; $n++){

			// Binario da entreparte apos endereco de rede
			// e antes do espaco para hosts
			$nbin = substr($zeros.decbin($n), -$diffbits);

			// Gerar binario da subnet
			$subbin = substr($netbits . $nbin . $zeros, 0, 32);

			// echo "#debug> N=$n netbits=$netbits Nbin=$nbin subbin=$subbin\n";
			$subnet = ipv4_bin2ip($subbin) . $pfxsuffix;
			$list[] = $subnet;

			// controle de limite
			$c++;
			if($limit && $c >= $limit) break;
		}

		return $list;
	}


	// Casamento de prefixos em netmap em listas de prefixos
	// de tamanhos de prefixos diferentes
	function &ipv4_prefixes_wedding($key_prefixes, $vle_prefixes){
		$map = array();
		// Argumentos precisam ser listas
		if(!is_array($key_prefixes) || !is_array($vle_prefixes)) return $map;

		$key_count = count($key_prefixes);
		$vle_count = count($vle_prefixes);
		// Listas nao podem estar vazias
		if($key_count < 1 || $vle_count < 1) return $map;

		// Localizar menor tamanho de prefixo
		$key_min_plen = 0; $key_max_plen = 0;
		$vle_min_plen = 0; $vle_max_plen = 0;
		$a=''; $b='';
		$key_list = array();
		$vle_list = array();
		// Prefixos chave
		foreach($key_prefixes as $k=>$v){
			list($a, $b) = explode('/', $v);
			$b = (int)$b;
			if($key_min_plen < 1) $key_min_plen = $b;
			if($key_max_plen < 1) $key_max_plen = $b;
			$key_min_plen = min($key_min_plen, $b);
			$key_max_plen = max($key_max_plen, $b);
			//echo "# key -- a=$a b=$b\n";
			// ignorar prefixos pequenos quando um prefixo maior existir
			if(isset($key_list[$a]) && $key_list[$a] > $b) continue;
			$key_list[$a] = $b;
		}
		// Prefixos valor
		foreach($vle_prefixes as $k=>$v){
			list($a, $b) = explode('/', $v);
			$b = (int)$b;
			if($vle_min_plen < 1) $vle_min_plen = $b;
			if($vle_max_plen < 1) $vle_max_plen = $b;
			$vle_min_plen = min($vle_min_plen, $b);
			$vle_max_plen = max($vle_max_plen, $b);
			//echo "# vle -- a=$a b=$b\n";
			// ignorar prefixos pequenos quando um prefixo maior existir
			if(isset($vle_list[$a]) && $vle_list[$a] > $b) continue;
			$vle_list[$a] = $b;
		}
		$max_plen = max($key_max_plen, $vle_max_plen);

		// Debug
		// echo "# [debug]\n";
		// echo "# key_min_plen...: $key_min_plen\n";
		// echo "# key_max_plen...: $key_max_plen\n";
		// echo "# vle_min_plen...: $vle_min_plen\n";
		// echo "# vle_max_plen...: $vle_max_plen\n";
		// echo "# max_plen.......: $max_plen\n";
		// echo "#\n";
		// echo "# key_list :: "; print_r($key_list);
		// echo "# vle_list :: "; print_r($vle_list);
		// echo "#\n";

		// Gerar nova lista baseado nas listas atuais quebrando
		// pelo tamanho de prefixos mais especifico entre as listas
		$key_subnets = array();
		$id = 1;
		foreach($key_list as $pfx=>$plen){
			$prefix = $pfx.'/'.$plen;
			$subnets = ipv4_get_subnets($prefix, $max_plen, 1);
			foreach($subnets as $k=>$snet) $key_subnets[$id++] = $snet;
		}
		$vle_subnets = array();
		$id = 1;
		foreach($vle_list as $pfx=>$plen){
			$prefix = $pfx.'/'.$plen;
			$subnets = ipv4_get_subnets($prefix, $max_plen, 1);
			foreach($subnets as $k=>$snet) $vle_subnets[$id++] = $snet;
		}

		// echo "# key_subnets :: "; print_r($key_subnets);
		// echo "# vle_subnets :: "; print_r($vle_subnets);
		// echo "#\n";

		// Combinar arrays
		$kcount = count($key_subnets);
		$vcount = count($vle_subnets);
		$tcount = min($kcount, $vcount);

		// echo "# kcount....: $kcount\n";
		// echo "# vcount....: $vcount\n";
		// echo "# tcount....: $tcount\n";
		// echo "#\n";

		$id = 1;
		for($i=1; $i <= $tcount; $i++){
			$k = $key_subnets[$i];
			$v = $vle_subnets[$i];
			$map[$k] = $v;
		}
		return $map;
	}


	// Contar numero de IPs totais em um prefixo
	function ipv4_prefix_count($ipv4_prefix){
		$c = 1;

		$p = strpos($ipv4_prefix, '/');
		if($p!==false){
			$bits = (int)substr($ipv4_prefix, $p+1);
			if($bits){
				$c = pow(2, 32-$bits);
			}
		}

		return $c;
	}


	// Verificar se um IP esta contido em uma subrede (teste de ip em uma ACL/prefixo)
	// Esta contigo? sim=true, nao=false
	function ipv4_addr_test_prefix($ipv4_addr, $ipv4_prefix){
		$r = 0;

		// Isolar IP sem prefixo
		$d = strpos($ipv4_addr, '/');
		if($d!==false) $ipv4_addr = substr($ipv4_addr, 0, $d);

		// Prefixo requer prefixo
		$d = strpos($ipv4_prefix, '/');
		if($d===false) $ipv4_prefix .= '/32';


		// Carregar informacoes do prefixo
		$ipv4_prefix_info =& ipv4_load($ipv4_prefix);
		$network = $ipv4_prefix_info['network'];
		$broadcast = $ipv4_prefix_info['broadcast'];

		// Converter para 32 bits numerico
		// - primeiro IP
		$_a = ip2long($network);
		// - Ultimo IP
		$_b = ip2long($broadcast);
		// - IP a ser testado
		$_i = ip2long($ipv4_addr);

		// Se o IP a ser testado estiver entre o primeiro e o ultimo, ele
		// pertence ao prefixo
		if($_a <= $_i && $_b >= $_i) $r = 1;

		//echo "    -- ipv4_addr_test_prefix($ipv4_addr, $ipv4_prefix) network=$network($_a) broadcast=$broadcast($_b) ipv4_addr=$ipv4_addr ($_i) -- R=$r\n";

		//print_r($ipv4_prefix_info);
		$netbits = $ipv4_prefix_info['netbits'];

		return $r;
	}





?>