<?php

	// Biblioteca de analise de enderecos ipv6
	
	// Retornar endereco ou prefixo ipv6 em formato completo - todos os zeros
	/*
	$list = array(
		'2001:db8:beba:cafe::2/64',
		'2001:db8:beba:cafe::1',
		'2001:db8::/32',
		'2000::/3',
		'::/0'
	);
	foreach ($list as $key => $value) {
		$output = ipv6_full($value);
		echo "# ($value) => [$output]\n";
	}
	exit();
	*/
	function ipv6_full($ipv6in){
		$ipv6out = trim($ipv6in);
		// verificar se e' preixo
		$plen = -1;
		$p = strpos($ipv6out, '/');
		if($p !== false){
			$plen = (int)substr($ipv6out, $p+1);
			$ipv6out = substr($ipv6out, 0, $p);
		}
		// echo "A> IN=$ipv6in, IPv6=$ipv6out, PLEN=$plen\n";
		$w = ipv6_load($ipv6out,$plen);
		$ipv6out = $w['addressfull'];
		if($plen >= 0) $ipv6out .= '/' . $plen;
		// echo "B> OUTPUT IPv6=$ipv6out\n";
		return $ipv6out;
	}

	// ler endereco ipv6
	function &ipv6_read($ipv6){
		$info = array('valid' => false, 'input' => $ipv6, 'tokens'=>array(), 'tcount' => 0);
		$len = strlen($ipv6);
		$str = '';
		$jumpcount = 0;
		for($c=0; $c<$len; $c++){
			$chr = substr($ipv6, $c, 1);
			$ord = ord($chr);

			// 48-57 65-70 97-102
			if( ! ( $chr==':' || ($ord >= 48 && $ord <= 57) ||  ($ord >= 65 && $ord <= 70) || ($ord >= 97 && $ord <= 102) ) ){
				$info['errno'] = 1;
				$info['error'] = 'prohibited byte';
				return $info;
			}
			
			$dbl = substr($ipv6, $c, 2);
			// divisores invalidos
			if($dbl=='.:'||$dbl==':.'||$dbl=='..'){$info['errno'] = 3;$info['error'] = 'wrong sep found';return $info;}
			// divisores
			if($dbl=='::'){
				if($str!=''){ $info['tokens'][] = array(0 => $str, 1 => 1); $str=''; }
				if($jumpcount){$info['errno'] = 4;$info['error'] = 'double jump found';return $info;}
				$jumpcount++;
				$info['tokens'][]=array(0=>'::', 1=>4);
				$c++;
				continue;
			}
			if($chr==':'){
				if($str!=''){ $info['tokens'][] = array(0 => $str, 1 => 1); $str=''; }
				$info['tokens'][]=array(0=>':', 1=>3); continue;
			}
			// numeros
			$str .= $chr;
			//echo "CHR: $chr  DBL: $dbl\n";
		}
		if($str!=''){ $info['tokens'][] = array(0 => $str, 1 => 1); $str=''; }
		$info['tcount'] = count($info['tokens']);

		// sem saltos, mas nao tem todas as casas
		if(!$jumpcount && $info['tcount'] < 15){ $info['errno'] = 1; $info['error'] = 'less than 16 bytes'; return $info; }
	
		// excesso de numeros
		if($info['tcount']>15){ $info['errno'] = 5; $info['error'] = 'tokens overflow'; return $info; }
		
		// qualquer sequencia maior que 4 bytes
		foreach($info['tokens'] as $k=>$token)
			if(strlen($token[0])>4){ $info['errno'] = 6; $info['error'] = 'more than 4 bytes'; return $info; }
		//-
		$info['valid'] = 1;
		return $info;
	}
	

	// Leitura IPv6 completa, completando todas as 16 casas e os zeros a esquerda
	function ipv6fullread_parts($ipv6addr){
		$m = strpos($ipv6addr, '/');
		if($m!==false) $ipv6addr = substr($ipv6addr, 0, $m);

		$list = array();
		for($i=0; $i<=7; $i++) $list[$i] = '0000';

		$forward_list = array();
		$backward_list = array();

		// Quebrar em partes
		$parts = explode(':', $ipv6addr);
		$c = count($parts);
		$idx = 0;
		$direct = 1;
		//echo "# INPUT: [$ipv6addr] C [$c]\n";
		foreach($parts as $k=>$part){
			$zpart = substr('0000'.$part, -4);
			//echo "# - PART=[$part] ZPART=[$zpart] direct=[$direct] idx=[$idx] C=[$c] K=[$k]\n";
			// Espaco duplo
			if($part==''){
				$direct = 0;
				$idx = 7;
				continue;
			}
			if($direct){
				$forward_list[] = $zpart;
			}else{
				$backward_list[] = $zpart;
			}
		}
		$forward_count  = count($forward_list);
		$backward_count = count($backward_list);

		// Primeiro as partes na sequencia
		$fdx = 0;
		if($forward_count) for($fdx=0; $fdx < $forward_count; $fdx++) $list[$fdx] = $forward_list[$fdx];
		//$fdx--;
		
		// Completar de traz pra frente com as partes finais
		$cdx = 0;
		$idx = 7;
		if($backward_count){
			// inverter array
			$last_idx = ($backward_count - 1);
			$c = 0;
			for($cdx=$backward_count -1; $cdx >= 0; $cdx--){
				//$k = ($backward_count-1) - $cdx;
				if($idx <= $fdx) break;
				$list[$idx] = $backward_list[$cdx];
				$idx--;
			}
		}

		// echo "forward_list  [$forward_count]..: "; print_r($forward_list);
		// echo "backward_list [$backward_count].: "; print_r($backward_list);

		return $list;
	}

	function ipv6fullread($ipv6addr){
		$parts = ipv6fullread_parts($ipv6addr);
		return implode(':', $parts);
	}


	// Converter binario em endereco IPv6
	function ipv6_bin2ip($bin){
		$bin = substr($bin . str_repeat('0', 128), 0, 128);

		$len = strlen($bin);
		$hexparts = array();
		for($i=0; $i<=7;$i++) $hexparts[$i]='0000';
		$idx = 0;

		// ler partes
		$hpart = '';
		$hidx = 0;
		$c = 0;
		while($c < $len){
			$part = substr($bin, $c, 4);
			$plen = strlen($part);
			if($plen < 4){
				// Completar cada 4 bits com zeros a direita
				$part = substr($part.'0000', 0, 4);
			}
			$hex = dechex(bindec($part));
			$hpart .= $hex;
			$idx++;

			// Completou 4 bytes hexadecimais
			if($idx % 4 === 0){
				$hparts[$hidx] = $hpart;
				$hpart = '';
				$hidx++;
			}
			$c += 4;
		}

		// Campo sobrando hexadecimal
		if($hpart!=''){
			$hpart = substr($hpart . '0000', 0, 4);
			$hparts[$hidx] = $hpart;
		}
		$ipv6full = implode(':', $hparts);

		return $ipv6full;
	}

	// Resumir endereco Ipv6 retirando zeros a esquerda e duplos campos brancos
	function ipv6_resume($ipv6){
		$parts = explode(':', $ipv6);

		$zero_index = array();
		$zero_point = 0;
		$zero_count = 0;
		$zero_found = 0;
		// $zero_maxpoint = 0;
		// $zero_maxcount = 0;

		// Resumir zeros a esquerda
		foreach($parts as $k=>$part){
			$spart = ltrim($part, '0');
			if($spart=='') $spart = '0';
			if($spart==='0'){
				// Zero encontrado
				if(!$zero_count){
					// Primeiro registro
					$zero_point = $k;
				}
				$zero_count++;
			}else{
				// Nao e' zero
				// Parar contador de zeros se encontrado
				// - Ponto valido
				if($zero_count > 1){
					$zero_index[$zero_point] = $zero_count;
					$zero_found++;
				}
				// Desconsiderar contagem
				$zero_point = 0;
				$zero_count = 0;
			}
			//echo "#  register [$k] = [$spart]\n";
			$parts[$k] = $spart;
		}

		// Finalizar contador de zeros no final do endereco
		if($zero_count){
			// Parar contador de zeros se encontrado
			if($zero_count > 1){
				// Ponto valido
				$zero_index[$zero_point] = $zero_count;
				$zero_found++;
			}
		}
		// Temos pontos a encurtar?
		if($zero_found){
			$zero_maxpoint = 0;
			$zero_maxcount = 0;
			foreach($zero_index as $k=>$v){
				if($v > $zero_maxcount){
					$zero_maxcount = $v;
					$zero_maxpoint = $k;
				}
			}
			// Ir para o ponto e apagar zeros seguintes
			$deletes = $zero_maxcount;
			for($k = $zero_maxpoint; $k <= 7; $k++){
				$part = $parts[$k];
				//echo "# DELETE POINT [$k] PART [$part]\n";
				if($deletes < 1) break;
				if($part==='0'){
					if($zero_maxpoint===$k){
						$parts[$k] = ':';
						$deletes--;
					}else{
						unset($parts[$k]);	
						$deletes--;
					}
				}else{
					// acabou
					break;
				}
			}
		}

		$ipv6r = implode(':', $parts);

		// echo "PARTS: "; print_r($parts);
		// echo "zero_index: "; print_r($zero_index);
		// echo "# INPUT....: ",$ipv6,"\n";
		// echo "# OUTPUT...: ",$ipv6r,"\n";

		// Correcao de gateway padrao
		if($ipv6r==':') $ipv6r = '::';

		// Tripla reducao
		$ipv6r = str_replace(':::', '::', $ipv6r);

		return $ipv6r;
	}

	// Resumir endereco de rede IPv6 de maneira a deixar cheio apenas os campos utilizados
	function ipv6_fullnet_resume($ipv6net, $plen=64){
		$ipv6net = trim($ipv6net);
		$d = strpos($ipv6net, '/');
		if($d!==false){
			$network = substr($ipv6net, 0, $d);
			$plen = (int)substr($ipv6net, $d+1);
		}else{
			$network = $ipv6net;
		}

		// Converter em grupos-partes
		$parts = explode(':', $network);
		// Completar partes com zeros
		foreach($parts as $k=>$v){
			$v = substr('0000'.$v, -4);
			$parts[$k] = $v;
		}
		//$plen = 41;

		$cutlen = $plen + ($plen % 4);
		$incparts = (int)(($plen / 16) + ( ($plen % 16) ? 1 : 0 ) );

		$finalparts = array();
		for($i=0; $i < $incparts; $i++) if(isset($parts[$i])) $finalparts[$i] = $parts[$i];
		$ipv6r = implode(':', $finalparts) . '::/' . $plen;
		$ipv6r = str_replace('::::', '::', $ipv6r);
		$ipv6r = str_replace(':::', '::', $ipv6r);
		return $ipv6r;

		// echo "ipv6net.......: ",$ipv6net,"\n";
		// echo "incparts......: ",$incparts,"\n";
		// echo "plen..........: ",$plen,"\n";
		// echo "cutlen........: ",$cutlen,"\n";
		// echo "PARTS.........: "; print_r($parts);
		// echo "FINALPARTS....: "; print_r($finalparts);
		// echo "\n";
		//exit();
	}

	// carrega informacoes sobre endereço e mascara
	function &ipv6_load($_ip, $_arg_prefixlen = 64){
		$_prefixlen = $_arg_prefixlen;

		// padrao da resposta
		$r = array(
			'address' => '',	// endereco passado em $_ip
			'prefixlen' => 0,	// bits da mascara
			'prefix' => '',		// prefixo final bem lido
			'global' => 0,		// 0=nao, 1=ip global
			'type' => 0,		// 0=net, 1=host
			'network' => '',	// endereco da rede com prefixo (abcd::/64)
			'bitsn' => 0,		// numero de bits na rede
			'bitsh' => 0,		// numero de bits no host
			'ipv' => 6 			// versao do protocolo ip
		);
		// carregar ip enviado
		$_ip = trim($_ip);

		// detectar presenca de mascara
		$_bp = strpos($_ip, '/');
		if($_bp!==false){
			$_prefixlen = (int)substr($_ip, $_bp+1);
			$_ip = substr($_ip, 0, $_bp);
		}
		if($_prefixlen < 0) $_prefixlen = 0;
		if($_prefixlen > 128) $_prefixlen = 128;

		$_prefixlen=(int)$_prefixlen;
		$_fip = ipv6fullread($_ip);
		$_rip = ipv6_resume($_fip);

		//echo "#** _fip=[$_fip] _ip=[$_ip] _prefixlen=[$_prefixlen] _rip=[$_rip]\n";

		$r['address'] = $_rip;
		$r['prefixlen'] = $_prefixlen;
		$r['bitsn'] = $_prefixlen;
		$r['bitsh'] = 128 - $_prefixlen;

		// Endereco e Prefixo completo
		$r['addressfull'] = $_fip;

		// Prefixo global comeca com 3 no primeiro byte
		$fc = substr($_fip, 0, 1);
		$r['global'] = ($fc == '2' || $fc == '3' ? 1 : 0);

		// Obter partes
		$partsh = explode(':', $_fip);
		$_fullhex = implode('', $partsh);
		$r['fullhex'] = $_fullhex;

		// Gerar grande binario
		$i = 0;
		$l = strlen($_fullhex);
		$fullbin = '';
		for($i = 0; $i < $l; $i+=2){
			$h = substr($_fullhex, $i, 2);
			$b = substr('0000000'.decbin(hexdec($h)), -8);
			$fullbin .= $b;
		}
		$r['fullbin'] = $fullbin;
		$netbits = substr($fullbin, 0, $_prefixlen);
		$r['netbits'] = $netbits;
		$r['hostbits'] = substr($fullbin, $_prefixlen);
		$fullnet = ipv6_bin2ip($r['netbits']);
		$r['fullnet'] = $fullnet;
		$r['network'] = ipv6_resume($fullnet) . '/' . $_prefixlen;

		// Gerar prefixo de rede
		$bits = substr($r['netbits'].str_repeat('0', 128), 0, 128);
		// converter para hexadecimal
		$i = 0;
		$l = strlen($bits);
		$fullhex = array();
		$x = 0;
		$hex = '';
		for($i = 0; $i < $l; $i+=8){
			$b = substr($bits, $i, 8);
			$h = substr('00'.dechex(bindec($b)), -2);
			$hex .= $h;
			$x++;
			if($x==2){
				$x = 0;
				$fullhex[] = $hex;
				$hex = '';
			}
			//echo "> I=$i L=$l H=$h B=[$b]\n";
		}
		$r['fullnethex'] = implode(':', $fullhex);
		// Resumir zeros pelo codigo em C do AF_INET6
		//$r['finaladdr'] = ipv6read($r['fullnethex']);
		$r['finaladdr'] = $r['fullnethex'];
		$r['fullprefix'] = $r['finaladdr'].'/'.$_prefixlen;
		$r['prefixfull'] = $r['fullprefix'];
		$r['fullnetwork'] = ipv6_fullnet_resume($r['network']);

		// Prefixo resumido
		$r['prefix'] = ipv6_resume($r['finaladdr']).'/'.$_prefixlen;

		if($_prefixlen==128){
			$r['type'] = 1;
		}else{
			$r['type'] = (str_replace('0', '', $r['hostbits']) == '' ? 0 : 1);
		}

		return $r;
	}

	// testar se ipv6 esta digitado corretamente
	function ipv6_test($addr){
		$read = ipv6_read($addr);
		return $read['valid'];
	}



	// Quebrar prefixo IPv6 em pedacos
	// Argumentos:
	// - prefix......: prefixo (::/nn)
	// - prefix_len..: tamanho do prefixo das partes geradas
	// - inc_plen....: incluir mascara nos sub-prefix
	// - limit.......: limitar numero de prefixos, 0=nao limitar
	//
	function &ipv6_get_subnets($prefix, $subnet_prefix_len=64, $inc_plen=1, $limit=0){
		$list = array();

		$ipinfo = ipv6_load($prefix);
		$prefixlen = $ipinfo['prefixlen'];
		$netbits = $ipinfo['netbits'];

		$pfxsuffix = ($inc_plen ? '/' . $subnet_prefix_len : '');


		//echo "DEBUG: IPINFO - pfxsuffix=$pfxsuffix, subnet_prefix_len=$subnet_prefix_len\n"; print_r($ipinfo);
		/*
			Array
			(
			    [address] => 2001:db8:a101::
			    [prefixlen] => 48
			    [prefix] => 2001:db8:a101::/48
			    [global] => 1
			    [type] => 0
			    [network] => 2001:db8:a101::/48
			    [bitsn] => 48
			    [bitsh] => 80
			    [ipv] => 6
			    [addressfull] => 2001:0db8:a101:0000:0000:0000:0000:0000
			    [fullhex] => 20010db8a10100000000000000000000
			    [fullbin] => 00100000000000010000110110111000101000010000000100000000000000000000000000000000000000000000000000000000000000000000000000000000
			    [netbits] => 001000000000000100001101101110001010000100000001
			    [hostbits] => 00000000000000000000000000000000000000000000000000000000000000000000000000000000
			    [fullnet] => 2001:0db8:a101:0000:0000:0000:0000:0000
			    [fullnethex] => 2001:0db8:a101:0000:0000:0000:0000:0000
			    [finaladdr] => 2001:0db8:a101:0000:0000:0000:0000:0000
			    [fullprefix] => 2001:0db8:a101:0000:0000:0000:0000:0000/48
			    [prefixfull] => 2001:0db8:a101:0000:0000:0000:0000:0000/48
			    [fullnetwork] => 2001:0db8:a101::/48
			)
		*/

		// Prefixo do tamanho certo
		if($prefixlen==$subnet_prefix_len){
			$list[] = $ipinfo['network'] . $pfxsuffix;
			// $list[] = $prefix;
			return $list;
		}

		// Prefixo menor que o pedaco solicitado
		if($prefixlen > $subnet_prefix_len) return $list;

		// Prefixo maior, calcular numero de pedacos possiveis
		$diffbits = $subnet_prefix_len - $prefixlen;
		// Nao calcular mais que 16 bits
		if($diffbits > 16) $diffbits = 16;
		$subcount = pow(2, $diffbits);

		$plimit = $limit;
		if(!$plimit) $plimit = $subcount;

		//echo "DEBUG: subcount=$subcount, diffbits=$diffbits, prefixlen=$prefixlen netbits=$netbits, limit=$limit plimit=$plimit\n";
		//exit();


		// Gerar subnets
		$zeros = str_repeat('0', 128);
		$c = 0;
		for($n = 0; $n < $subcount; $n++){

			// Binario da entreparte apos endereco de rede
			// e antes do espaco para hosts
			$nbin = substr($zeros.decbin($n), -$diffbits);

			// Gerar binario da subnet
			$subbin = substr($netbits . $nbin . $zeros, 0, 128);

			// echo "#debug> N=$n netbits=$netbits Nbin=$nbin subbin=$subbin\n";
			$subnet = ipv6_bin2ip($subbin) . $pfxsuffix;
			$list[] = $subnet;

			// controle de limite
			$c++;
			if($plimit && $c >= $plimit) break;
		}

		return $list;
	}



	// Verificar se um IP esta contido em uma subrede (teste de ip em uma ACL/prefixo)
	// Esta contigo? sim=true, nao=false
	function ipv6_addr_test_prefix($ipv6_addr, $ipv6_prefix){
		$r = false;

		// Isolar IP sem prefixo
		$d = strpos($ipv6_addr, '/');
		if($d!==false) $ipv6_addr = substr($ipv6_addr, 0, $d);

		// Prefixo requer prefixo
		$d = strpos($ipv6_prefix, '/');
		if($d===false) $ipv6_prefix .= '/128';

		// FALTA FAZER
		//echo "# ipv6_addr_test_prefix($ipv6_addr, $ipv6_prefix)\n";

		return $r;
	}





?>