<?php

/* -----------------------------------------------------------------------------------------
	biblioteca para leitura e escrita em arquivos de configuracao
----------------------------------------------------------------------------------------- */



//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


	// salvar array em arquivo de configuraçao
	/*
	
		salvar em arquivo de configuracao
		parametros:
			file - caminho para arquivo
			conf - array de configuracao
		
		retorno:
			0 - sucesso
			1 - erro ao gravar arquivo
			2 - conf nao é array
			3 - diretorio nao alcancavel
			4 - array vazio

	*/
	function libconf_save($_file, &$_conf){
		// conf nao é array
		if(!is_array($_conf)) return 2;
		// array vazio, apagar arquivo
		if(!count($_conf)) return 4;
		// pasta nao disponivel
		$_dir = dirname($_file);
		if(!file_exists($_dir) && !mkdir($_dir)) return 3;
		// gerar saida
		$_output='';
		foreach($_conf as $_vname=>$_vvalue){
			if(is_array($_vvalue)){
				$_new=array(); foreach($_vvalue as $_k=>$_v)$_new[]=$_k.'='.$_v; $_vvalue=implode(' ', $_new);
			}
			$_output.=$_vname.' '.$_vvalue."\n";
		}
		if(file_put_contents($_file, $_output)) return 0;
		return 1;
	}

	/*
		carregar arquivo de configuracao para array
		parametros:
			file - caminho do arquivo de configuracao
			def - array com chaves padroes e orientacoes de tipos
		retorno:
			array com chaves sendo variaveis e valores como valores!
			array vazio caso falhas na leitura

	*/

	// carrega arquivo de configuração para array
	function &libconf_load($_file, $_default = false){
		$_conf = array();
		// arquivo nao existe
		if($_file=='' || !is_file($_file)){
			if(is_array($_default)) return $_default;
			return $_conf;
		}
		// ler arquivo
		$_lines = file($_file);
		// processar linha a linha
		foreach($_lines as $_line){
			// limpeza basica
			$_line = trim($_line);
			// pular comentario
			if(substr($_line, 0, 1)=='#') continue;
			// remover tabs
			$_line = str_replace(chr(9), ' ', $_line);
			// procurar primeiro igual ou espaco
			$_fe = strpos($_line, '=');
			$_fs = strpos($_line, ' ');
			$_div = $_fe;
			if($_div===false) $_div = $_fs;
			// selecionar menor em caso de ambos
			if($_fe!==false && $_fs !== false) $_div = ($_fe<$_fs?$_fe:$_fs);
			if($_div===false) continue;

			// cataloga
			$_conf[substr($_line, 0, $_div)] = trim(substr($_line, $_div+1));
		}
		// aplicar defaults
		if(is_array($_default) && count($_default)){
			foreach($_default as $_k=>$_v){
				if(!isset($_conf[$_k])){
					// nao definida
					$_conf[$_k] = $_v;
				}else{
					// ajuste de tipo
					$_tb = gettype($_default[$_k]);
					if(gettype($_conf[$_k])!=$_tb) settype($_conf[$_k], $_tb);
				}
			}
		}
		
		return $_conf;
	}

	// carregar arquivo no formato ini
	function &parse_ini($_file, $_sess=true){
		$sn = 'global';
		$_conf = array();
		if($_sess) $_conf[$sn] = array();
		$_lines = file($_file);
		foreach($_lines as $_line){
			$_line = replace(chr(9), ' ', $_line);
			$_line = replace('  ', ' ', $_line);
			$_line = trim($_line);
			if($_line=='') continue;
			$fc = substr($_line, 0, 1);
			$lc = substr($_line, -1);
			if($_sess && $fc=='[' && $lc==']'){	// em modo sessao, procurar chave de sessao
				$_x = trim(substr($_line, 1, -1));
				if($_x!=''){
					$sn = $_x;
					if(!isset($_conf[$sn])) $_conf[$sn] = array();
					continue;
				}
			}
			// pular nao-variaveis
			$dp = strpos($_line, '=');
			if($dp===false) continue;
			$k=substr($_line, 0, $dp);
			$v=trim(substr($_line, $dp+1));
			if($v!=''){
				if($_sess){
					$_conf[$sn][$k]=$v;	
				}else{
					$_conf[$k]=$v;
				}
			}
		}
		return $_conf;
	}


	// carregar arquivo no formato ini
	function save_ini($_file, $_conf){
		$text = '';
		foreach($_conf as $_sess=>$_vars){
			$text .= '['.$_sess.']'."\n";
			foreach($_vars as $k=>$v) $text.=$k.'='.$v."\n";
			$text .= "\n";
		}
		return file_put_contents($_file, $text)?1:0;
	}

	// processa linha contendo pares de valores e variaveis e retorna array
// retornar array com chave=> valor de uma string contendo chave=valor em uma unica linha
	function libconf_line($str){
		$ret = array();
		$r = '';
		$r = str_replace("\r", "\n", $str); // remover quebras do windows
		$r = str_replace("\n\n", "\n", $r); // remover quebras duplicadas
		$r = str_replace("\n", " ", $r); // remover quebras
		$ar = explode(" ", $r);

		$key = '';
		foreach($ar as $line){
			$line = trim($line);
			if(substr($line, 0, 1)=='#') continue;
			$pos = strpos($line, '=');
			if($pos !== false){
				$key = substr($line, 0, $pos);
				$ret[$key] = substr($line, $pos+1);
			}elseif($pos===false && $key != ''){
				$ret[$key] .= " " . $line;
			}
		}
		foreach($ret as $k=>$v) $ret[$k] = trim($v);
		return $ret;
	}

	// arquivo de configuração com suporte a multiplas variaveis por linha



?>